@@ -1,7 +1,7 @@
 /*
  * jconv.c - alternative japanese code conversion routines
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: jconv.c,v 1.19 2004/09/15 00:37:12 shirok Exp $
+ *  $Id: jconv.c,v 1.22 2007/09/11 12:29:37 shirok Exp $
  */
 
 /* Some iconv() implementations don't support japanese character encodings,
@@ -49,8 +49,8 @@
 #include <ctype.h>
 #include "charconv.h"
 
-#define INCHK(n)   do{if (inroom < (n)) return INPUT_NOT_ENOUGH;}while(0)
-#define OUTCHK(n)  do{if (outroom < (n)) return OUTPUT_NOT_ENOUGH;}while(0)
+#define INCHK(n)   do{if ((int)inroom < (n)) return INPUT_NOT_ENOUGH;}while(0)
+#define OUTCHK(n)  do{if ((int)outroom < (n)) return OUTPUT_NOT_ENOUGH;}while(0)
 
 #define ERRP(n)    ((n)==INPUT_NOT_ENOUGH||(n)==OUTPUT_NOT_ENOUGH||(n)==ILLEGAL_SEQUENCE)
 
@@ -607,8 +607,8 @@ static size_t utf2eucj(ScmConvInfo *cinfo, const char *inptr, size_t inroom,
                        char *outptr, size_t outroom, size_t *outchars)
 {
     unsigned char u0;
-    
     u0 = (unsigned char)inptr[0];
+
     if (u0 <= 0x7f) {
         *outptr = u0;
         *outchars = 1;
@@ -1235,7 +1235,7 @@ static size_t jconv_1tier(ScmConvInfo *info, const char **iptr,
     ScmConvProc cvt = info->convproc[0];
     const char *inp = *iptr;
     char *outp = *optr;
-    int inr = *iroom, outr = *oroom; 
+    int inr = (int)*iroom, outr = (int)*oroom; 
     size_t outchars, inchars, converted = 0;
 
 #ifdef JCONV_DEBUG
@@ -1250,9 +1250,9 @@ static size_t jconv_1tier(ScmConvInfo *info, const char **iptr,
         } else {
             converted += inchars;
             inp += inchars;
-            inr -= inchars;
+            inr -= (int)inchars;
             outp += outchars;
-            outr -= outchars;
+            outr -= (int)outchars;
         }
     }
     *iptr = inp;
@@ -1272,7 +1272,7 @@ static size_t jconv_2tier(ScmConvInfo *info, const char **iptr, size_t *iroom,
     ScmConvProc ocvt = info->convproc[1];
     const char *inp = *iptr;
     char *outp = *optr;
-    int inr = *iroom, outr = *oroom;
+    int inr = (int)*iroom, outr = (int)*oroom;
     size_t outchars, inchars, bufchars, converted = 0;
 
 #ifdef JCONV_DEBUG
@@ -1295,9 +1295,9 @@ static size_t jconv_2tier(ScmConvInfo *info, const char **iptr, size_t *iroom,
         }
         converted += inchars;
         inp += inchars;
-        inr -= inchars;
+        inr -= (int)inchars;
         outp += outchars;
-        outr -= outchars;
+        outr -= (int)outchars;
     }
     *iptr = inp;
     *iroom = inr;
