@@ -1,7 +1,7 @@
 ;;;
 ;;; gdbm - gdbm interface
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: gdbm.scm,v 1.9 2005/05/11 02:48:03 shirok Exp $
+;;;  $Id: gdbm.scm,v 1.13 2007/10/23 20:23:29 shirok Exp $
 ;;;
 
 (define-module dbm.gdbm
@@ -42,11 +42,12 @@
           gdbm-firstkey      gdbm-nextkey        gdbm-reorganize
           gdbm-sync          gdbm-exists?        gdbm-strerror
           gdbm-setopt        gdbm-version        gdbm-file-of
+          gdbm-errno
           |GDBM_READER|      |GDBM_WRITER|       |GDBM_WRCREAT|
           |GDBM_NEWDB|       |GDBM_FAST|         |GDBM_SYNC|
           |GDBM_NOLOCK|      |GDBM_INSERT|       |GDBM_REPLACE|
-          |GDBM_CACHESIZE|   |GDBM_FASTMODE|     |GDBM_CENTFREE|
-          |GDBM_COALESCEBLKS|)
+          |GDBM_CACHESIZE|   |GDBM_FASTMODE|     |GDBM_SYNCMODE|
+          |GDBM_CENTFREE|    |GDBM_COALESCEBLKS|)
   )
 (select-module dbm.gdbm)
 (dynamic-load "gdbm")
@@ -154,9 +155,7 @@
 
 (define (%with-gdbm-locking path thunk)
   (let1 db (gdbm-open path 0 |GDBM_READER| #o664) ;; put read-lock
-    (with-error-handler
-        (lambda (e) (gdbm-close db) (raise e))
-      (lambda () (thunk) (gdbm-close db)))))
+    (unwind-protect (thunk) (gdbm-close db))))
         
 (define-method dbm-db-exists? ((class <gdbm-meta>) name)
   (file-exists? name))
