@@ -1,7 +1,7 @@
 /*
  * threads.h - Gauche threads support
  *
- *   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,12 +30,17 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: threads.h,v 1.4 2006/10/08 05:00:30 shirok Exp $
+ *  $Id: threads.h,v 1.6 2007/09/14 11:17:41 shirok Exp $
  */
 
 #ifndef GAUCHE_THREADS_H
 #define GAUCHE_THREADS_H
 
+#if defined(EXTTHREADS_EXPORTS)
+#define LIBGAUCHE_EXT_BODY
+#endif
+#include <gauche/extern.h>      /* redefine SCM_EXTERN */
+
 /*---------------------------------------------------------
  * Scheme-level thread API
  */
@@ -46,6 +51,33 @@ extern ScmObj Scm_ThreadJoin(ScmVM *vm, ScmObj timeout, ScmObj timeoutval);
 extern ScmObj Scm_ThreadSleep(ScmObj timeout);
 extern ScmObj Scm_ThreadTerminate(ScmVM *vm);
 
+/*---------------------------------------------------------
+ * Thread exception classes.
+ * NB: The actual structure, ScmThreadException, is defined in
+ * src/gauche/exception.h.
+ */
+
+SCM_CLASS_DECL(Scm_ThreadExceptionClass);
+#define SCM_CLASS_THREAD_EXCEPTION  (&Scm_ThreadExceptionClass)
+#define SCM_THREAD_EXCEPTION_P(obj) SCM_ISA(obj, SCM_CLASS_THREAD_EXCEPTION)
+#define SCM_THREAD_EXCEPTION(obj)   ((ScmThreadException*)(obj))
+
+SCM_CLASS_DECL(Scm_JoinTimeoutExceptionClass);
+#define SCM_CLASS_JOIN_TIMEOUT_EXCEPTION (&Scm_JoinTimeoutExceptionClass)
+#define SCM_JOIN_TIMEOUT_EXCEPTION_P     SCM_ISA(obj, SCM_CLASS_JOIN_TIMEOUT_EXCEPTION)
+
+SCM_CLASS_DECL(Scm_AbandonedMutexExceptionClass);
+#define SCM_CLASS_ABANDONED_MUTEX_EXCEPTION (&Scm_AbandonedMutexExceptionClass)
+#define SCM_ABANDONED_MUTEX_EXCEPTION_P     SCM_ISA(obj, SCM_CLASS_ABANDONED_MUTEX_EXCEPTION)
+
+SCM_CLASS_DECL(Scm_TerminatedThreadExceptionClass);
+#define SCM_CLASS_TERMINATED_THREAD_EXCEPTION (&Scm_TerminatedThreadExceptionClass)
+#define SCM_TERMINATED_THREAD_EXCEPTION_P     SCM_ISA(obj, SCM_CLASS_TERMINATED_THREAD_EXCEPTION)
+
+SCM_CLASS_DECL(Scm_UncaughtExceptionClass);
+#define SCM_CLASS_UNCAUGHT_EXCEPTION (&Scm_UncaughtExceptionClass)
+#define SCM_UNCAUGHT_EXCEPTION_P     SCM_ISA(obj, SCM_CLASS_UNCAUGHT_EXCEPTION)
+
 /*---------------------------------------------------------
  * SYNCHRONIZATION DEVICES
  *
