@@ -1,7 +1,8 @@
 ;;;
 ;;; Array operations.  This is superset of SRFI-25.
 ;;;
-;;;  Copyright(C) 2002 by Shiro Kawai (shiro@acm.org)
+;;;  Copyright(C) 2002-2007 by Shiro Kawai (shiro@acm.org)
+;;;  Copyright(C) 2004      by Alex Shinn  (foof@synthcode.com)
 ;;;
 ;;;  Permission to use, copy, modify, distribute this software and
 ;;;  accompanying documentation for any purpose is hereby granted,
@@ -12,7 +13,7 @@
 ;;;  warranty.  In no circumstances the author(s) shall be liable
 ;;;  for any damages arising out of the use of this software.
 ;;;
-;;;  $Id: array.scm,v 1.10 2003/04/25 03:38:45 foof Exp $
+;;;  $Id: array.scm,v 1.17 2007/02/21 04:56:41 shirok Exp $
 ;;;
 
 ;; Conceptually, an array is a backing storage and a procedure to
@@ -25,9 +26,11 @@
   (use gauche.collection)
   (use gauche.sequence)
   (export <array-meta> <array>
+          <u8array> <s8array> <u16array> <s16array> <u32array> <s32array>
+          <u64array> <s64array> <f16array> <f32array> <f64array>
           array? make-array shape array array-rank
           array-start array-end array-ref array-set!
-          share-array subarray
+          share-array subarray array-equal?
           array-valid-index?  shape-valid-index?
           array-shape array-length array-size
           array-for-each-index shape-for-each
@@ -35,19 +38,50 @@
           array-for-each array-every array-any
           tabulate-array array-retabulate!
           array-map array-map! array->vector array->list
-          )
-  )
+          make-u8array make-s8array make-u16array make-s16array
+          make-u32array make-s32array make-u64array make-s64array
+          make-f16array make-f32array make-f64array
+          array-concatenate array-transpose array-rotate-90 array-flip array-flip!
+          identity-array array-inverse determinant determinant! array-mul array-expt
+          array-div-left array-div-right array-add-elements array-add-elements!
+          array-sub-elements array-sub-elements! array-mul-elements array-mul-elements!
+          array-div-elements array-div-elements! pretty-print-array
+          ))
 (select-module gauche.array)
 
+(autoload "gauche/matrix"
+  array-concatenate array-transpose array-rotate-90 array-flip array-flip!
+  identity-array array-inverse determinant determinant! array-mul array-expt
+  array-div-left array-div-right array-add-elements array-add-elements!
+  array-sub-elements array-sub-elements! array-mul-elements array-mul-elements!
+  array-div-elements array-div-elements! pretty-print-array)
+
 (define-class <array-meta> (<class>)
   ((backing-storage-creator :init-keyword :backing-storage-creator
                             :getter backing-storage-creator-of)
    (backing-storage-getter  :init-keyword :backing-storage-getter
                             :getter backing-storage-getter-of)
    (backing-storage-setter  :init-keyword :backing-storage-setter
-                            :getter backing-storage-setter-of))
+                            :getter backing-storage-setter-of)
+   (backing-storage-length  :init-keyword :backing-storage-length
+                            :getter backing-storage-length-of))
   )
 
+;; auto reader and writer for array-meta
+
+(define-method initialize ((class <array-meta>) initargs)
+  (next-method)
+  (let ((name (class-name class)))
+    (when name
+      (define-method write-object ((self class) port)
+        (format port "#,(~A ~S" name (array->list (array-shape self)))
+        (array-for-each (cut format port " ~S" <>) self)
+        (format port ")"))
+      (define-reader-ctor name
+        (lambda (sh . inits)
+          (list-fill-array! (make-array-internal class (apply shape sh)) inits))))))
+
+
 (define-class <array-base> ()
   ((start-vector    :init-keyword :start-vector :getter start-vector-of)
    (end-vector      :init-keyword :end-vector   :getter end-vector-of)
@@ -73,7 +107,85 @@
   ()
   :backing-storage-creator make-vector
   :backing-storage-getter vector-ref
-  :backing-storage-setter vector-set!)
+  :backing-storage-setter vector-set!
+  :backing-storage-length vector-length)
+
+(define-class <u8array> (<array-base>)
+  ()
+  :backing-storage-creator make-u8vector
+  :backing-storage-getter u8vector-ref
+  :backing-storage-setter u8vector-set!
+  :backing-storage-length u8vector-length)
+
+(define-class <s8array> (<array-base>)
+  ()
+  :backing-storage-creator make-s8vector
+  :backing-storage-getter s8vector-ref
+  :backing-storage-setter s8vector-set!
+  :backing-storage-length s8vector-length)
+
+(define-class <u16array> (<array-base>)
+  ()
+  :backing-storage-creator make-u16vector
+  :backing-storage-getter u16vector-ref
+  :backing-storage-setter u16vector-set!
+  :backing-storage-length u16vector-length)
+
+(define-class <s16array> (<array-base>)
+  ()
+  :backing-storage-creator make-s16vector
+  :backing-storage-getter s16vector-ref
+  :backing-storage-setter s16vector-set!
+  :backing-storage-length s16vector-length)
+
+(define-class <u32array> (<array-base>)
+  ()
+  :backing-storage-creator make-u32vector
+  :backing-storage-getter u32vector-ref
+  :backing-storage-setter u32vector-set!
+  :backing-storage-length u32vector-length)
+
+(define-class <s32array> (<array-base>)
+  ()
+  :backing-storage-creator make-s32vector
+  :backing-storage-getter s32vector-ref
+  :backing-storage-setter s32vector-set!
+  :backing-storage-length s32vector-length)
+
+(define-class <u64array> (<array-base>)
+  ()
+  :backing-storage-creator make-u64vector
+  :backing-storage-getter u64vector-ref
+  :backing-storage-setter u64vector-set!
+  :backing-storage-length u64vector-length)
+
+(define-class <s64array> (<array-base>)
+  ()
+  :backing-storage-creator make-s64vector
+  :backing-storage-getter s64vector-ref
+  :backing-storage-setter s64vector-set!
+  :backing-storage-length s64vector-length)
+
+(define-class <f16array> (<array-base>)
+  ()
+  :backing-storage-creator make-f16vector
+  :backing-storage-getter f16vector-ref
+  :backing-storage-setter f16vector-set!
+  :backing-storage-length f16vector-length)
+
+(define-class <f32array> (<array-base>)
+  ()
+  :backing-storage-creator make-f32vector
+  :backing-storage-getter f32vector-ref
+  :backing-storage-setter f32vector-set!
+  :backing-storage-length f32vector-length)
+
+(define-class <f64array> (<array-base>)
+  ()
+  :backing-storage-creator make-f64vector
+  :backing-storage-getter f64vector-ref
+  :backing-storage-setter f64vector-set!
+  :backing-storage-length f64vector-length)
 
 (define-method copy-object ((self <array-base>))
   (make (class-of self)
@@ -85,19 +197,28 @@
 ;; NB: these should be built-in; but here for now.
 (define-method copy-object ((self <vector>))
   (vector-copy self))
-
-;; reader and writer
-
-(define-method write-object ((self <array-base>) port)
-  (format port "#,~s"
-          (list* (class-name (class-of self))
-                 (array->list (array-shape self))
-                 (array->list self))))
-
-;; TODO: register read-ctor when subclass of <array-meta> is initialized.
-(define-reader-ctor '<array>
-  (lambda (sh . inits)
-    (apply array (apply shape sh) inits)))
+(define-method copy-object ((self <u8vector>))
+  (u8vector-copy self))
+(define-method copy-object ((self <s8vector>))
+  (s8vector-copy self))
+(define-method copy-object ((self <u16vector>))
+  (u16vector-copy self))
+(define-method copy-object ((self <s16vector>))
+  (s16vector-copy self))
+(define-method copy-object ((self <u32vector>))
+  (u32vector-copy self))
+(define-method copy-object ((self <s32vector>))
+  (s32vector-copy self))
+(define-method copy-object ((self <u64vector>))
+  (u64vector-copy self))
+(define-method copy-object ((self <s64vector>))
+  (s64vector-copy self))
+(define-method copy-object ((self <f16vector>))
+  (f16vector-copy self))
+(define-method copy-object ((self <f32vector>))
+  (f32vector-copy self))
+(define-method copy-object ((self <f64vector>))
+  (f64vector-copy self))
 
 ;;-------------------------------------------------------------
 ;; Affine mapper
@@ -149,21 +270,21 @@
 
 ;; shape index tests
 
-(define-method shape-valid-index? ((sh <array>) (ind <s32vector>))
+(define-method shape-valid-index? ((sh <array-base>) (ind <s32vector>))
   (let*-values (((Vb Ve) (shape->start/end-vector sh)))
     (and
      (= (s32vector-length ind) (s32vector-length Vb))
      (not (s32vector-range-check ind Vb (s32vector-sub Ve 1))))))
 
-(define-method shape-valid-index? ((sh <array>) (ind <vector>))
+(define-method shape-valid-index? ((sh <array-base>) (ind <vector>))
   (shape-valid-index? sh (vector->s32vector ind)))
-(define-method shape-valid-index? ((sh <array>) (ind <pair>))
+(define-method shape-valid-index? ((sh <array-base>) (ind <pair>))
   (shape-valid-index? sh (list->s32vector ind)))
-(define-method shape-valid-index? ((sh <array>) (ind <array>))
+(define-method shape-valid-index? ((sh <array-base>) (ind <array>))
   (shape-valid-index? sh (vector->s32vector (array->vector ind))))
-(define-method shape-valid-index? ((sh <array>) (ind <integer>) . more-index)
+(define-method shape-valid-index? ((sh <array-base>) (ind <integer>) . more-index)
   (shape-valid-index? sh (list->s32vector (cons ind more-index))))
-(define-method shape-valid-index? ((sh <array>))
+(define-method shape-valid-index? ((sh <array-base>))
   ;; special case - zero dimensional array
   (null? (array->list sh)))
 
@@ -196,7 +317,7 @@
         (unless (and (exact? (cadr l)) (integer? (cadr l)))
           (error "exact integer required for shape, but got" (cadr l)))
         (unless (<= (car l) (cadr l))
-          (error "beginning index ~s is larger than ending index ~s in shape argument: ~s" (car l) (cadr l) args)))
+          (errorf "beginning index ~s is larger than ending index ~s in shape argument: ~s" (car l) (cadr l) args)))
       ;; make array.
       (make <array>
         :start-vector (s32vector 0 0)
@@ -222,27 +343,54 @@
 ;; Make general array
 ;;
 
-(define (make-array shape . maybe-init)
+(define (make-array-internal class shape . maybe-init)
   (receive (Vb Ve) (shape->start/end-vector shape)
-    (make <array>
+    (make class
       :start-vector Vb
       :end-vector Ve
       :mapper (generate-amap Vb Ve)
-      :backing-storage (apply make-vector
+      :backing-storage (apply (backing-storage-creator-of class)
                               (fold * 1 (s32vector-sub Ve Vb))
                               maybe-init))))
 
-(define (array shape . inits)
-  (let* ((a   (make-array shape))
-         (bv  (backing-storage-of a))
-         (len (vector-length bv)))
+(define (make-array shape . opt)
+  (apply make-array-internal <array> shape opt))
+(define (make-u8array shape . opt)
+  (apply make-array-internal <u8array> shape opt))
+(define (make-s8array shape . opt)
+  (apply make-array-internal <s8array> shape opt))
+(define (make-u16array shape . opt)
+  (apply make-array-internal <u16array> shape opt))
+(define (make-s16array shape . opt)
+  (apply make-array-internal <s16array> shape opt))
+(define (make-u32array shape . opt)
+  (apply make-array-internal <u32array> shape opt))
+(define (make-s32array shape . opt)
+  (apply make-array-internal <s32array> shape opt))
+(define (make-u64array shape . opt)
+  (apply make-array-internal <u64array> shape opt))
+(define (make-s64array shape . opt)
+  (apply make-array-internal <s64array> shape opt))
+(define (make-f16array shape . opt)
+  (apply make-array-internal <f16array> shape opt))
+(define (make-f32array shape . opt)
+  (apply make-array-internal <f32array> shape opt))
+(define (make-f64array shape . opt)
+  (apply make-array-internal <f64array> shape opt))
+
+(define (list-fill-array! a inits)
+  (let* ((bv  (backing-storage-of a))
+         (set (backing-storage-setter-of (class-of a)))
+         (len ((backing-storage-length-of (class-of a)) bv)))
     (unless (= (length inits) len)
-      (errorf "array element initializer doesn't match the shape ~s: ~s"
-              (array->list shape) inits))
+      (errorf "array element initializer doesn't match the shape ~s [~s]: ~s"
+              (array->list (array-shape a)) len inits))
     (do ((i 0 (+ i 1))
          (inits inits (cdr inits)))
         ((= i len) a)
-      (vector-set! bv i (car inits)))))
+      (set bv i (car inits)))))
+
+(define (array shape . inits) (list-fill-array! (make-array shape) inits))
 
 (define (subarray ar sh)
   (let*-values (((Vb Ve) (shape->start/end-vector sh)))
@@ -351,7 +499,7 @@
                 ((constants coeffs)
                  (affine-proc->coeffs proc (size-of Vb)))
                 )
-    (make <array>
+    (make (class-of array)
       :start-vector Vb
       :end-vector   Ve
       :mapper (generate-shared-map (mapper-of array) constants coeffs)
@@ -375,8 +523,9 @@
 (define (array-size ar)
   (reduce * 1 (map (cute array-length ar <>) (iota (array-rank ar)))))
 
-(define-method object-equal? ((a <array>) (b <array>))
-  (let ((r   (array-rank a)))
+(define (array-equal? a b . opt)
+  (let ((eq (get-optional opt equal?))
+        (r (array-rank a)))
     (and (= r (array-rank b))
          (every (lambda (dim)
                   (and (= (array-start a dim) (array-start b dim))
@@ -385,13 +534,16 @@
          (call/cc
           (lambda (break)
             (array-for-each-index a
-                                  (lambda (index)
-                                    (unless (equal? (array-ref a index)
-                                                    (array-ref b index))
-                                      (break #f)))
-                                  (make-vector r 0))
+              (lambda (index)
+                (unless (eq (array-ref a index)
+                            (array-ref b index))
+                  (break #f)))
+              (make-vector r))
             #t)))))
 
+(define-method object-equal? ((a <array-base>) (b <array-base>))
+  (array-equal? a b equal?))
+
 ;; returns a proc that applies proc to indices that is given by a vector.
 
 (define (array-index-applier rank)
@@ -500,27 +652,17 @@
 (define (shape-for-each sh proc . o)
   (let* ((rank (array-end sh 0))
          (ser  (iota rank)))
-    (array-for-each-int proc
-                        ser
-                        (map-to <s32vector> (cut array-ref sh <> 0) ser)
-                        (map-to <s32vector> (cut array-ref sh <> 1) ser)
-                        o)))
+    (array-for-each-int
+     proc
+     ser
+     (map-to <s32vector> (cut array-ref sh <> 0) ser)
+     (map-to <s32vector> (cut array-ref sh <> 1) ser)
+     o)))
 
-(define (tabulate-array sh proc . o)
-  (let1 ar (make-array sh)
-    (cond
-     ((null? o)
-      (let ((iv (make-vector (array-end sh 0)))
-            (applier (array-index-applier (array-end sh 0))))
-        (shape-for-each sh (lambda (ind)
-                                   (array-set! ar ind (applier proc ind)))
-                              iv)))
-     ((or (vector? (car o)) (array? (car o)))
-      (shape-for-each sh (lambda (ind)
-                                 (array-set! ar ind (proc ind)))
-                            (car o)))
-     (else "bad index object (vector or array required)" (car o)))
-    ar))
+(define (tabulate-array sh . args)
+  (let ((res (make-array sh)))
+    (apply array-retabulate! res args)
+    res))
 
 ;; Mapping onto array.
 ;;   array-retabulate!
@@ -532,69 +674,70 @@
 ;; implementation it's not much of use.  I keep it just for compatibility
 ;; to Jussi's.
 
-(define-method array-retabulate! ((ar <array>) (sh <array>) (proc <procedure>) . o)
+(define-method array-retabulate! ((ar <array-base>) (sh <array-base>) (proc <procedure>) . o)
   ;; need to check the shape sh matches the ar's shape.
   (apply array-retabulate! ar proc o))
 
-(define-method array-retabulate! ((ar <array>) (proc <procedure>) . o)
-  (cond ((null? o)
-         (let ((applier (array-index-applier (array-rank ar))))
+(define-method array-retabulate! ((ar <array-base>) (proc <procedure>) . o)
+  (let ((set (backing-storage-setter-of (class-of ar)))
+        (store (backing-storage-of ar))
+        (mapper (mapper-of ar)))
+    (cond ((null? o)
+           (let ((applier (array-index-applier (array-rank ar))))
+             (array-for-each-index ar
+               (lambda (ind)
+                 (set store (mapper ind) (applier proc ind)))
+               (make-vector (array-rank ar)))))
+          ((or (vector? (car o)) (array? (car o)))
            (array-for-each-index ar
-                                 (lambda (ind)
-                                   (array-set! ar ind
-                                               (applier proc ind)))
-                                 (make-vector (array-rank ar)))))
-        ((or (vector? (car o)) (array? (car o)))
-         (array-for-each-index ar
-                               (lambda (ind)
-                                 (array-set! ar ind (proc ind)))
-                               (car o)))
-        (else "bad index object (vector or array required)" (car o))))
+             (lambda (ind)
+               (set store (mapper ind) (proc ind)))
+             (car o)))
+          (else "bad index object (vector or array required)" (car o)))))
 
-(define-method array-map! ((ar <array>) (sh <array>) (proc <procedure>) ar0 . more-arrays)
+(define-method array-map! ((ar <array-base>) (sh <array-base>) (proc <procedure>) ar0 . more-arrays)
   ;; need to check the shape sh matches the ar's shape.
   (apply array-map! ar proc ar0 more-arrays))
 
-(define-method array-map! ((ar <array>) (proc <procedure>) ar0)
-  (array-for-each-index ar
-                        (lambda (ind)
-                          (array-set! ar ind (proc (array-ref ar0 ind)))
-                          (make-vector (array-rank ar)))))
+(define-method array-map! ((ar <array-base>) (proc <procedure>) ar0)
+  (let ((set (backing-storage-setter-of (class-of ar)))
+        (store (backing-storage-of ar))
+        (mapper (mapper-of ar))
+        (get0 (backing-storage-getter-of (class-of ar0)))
+        (store0 (backing-storage-of ar0))
+        (mapper0 (mapper-of ar0)))
+    (array-for-each-index ar0
+      (lambda (ind)
+        (set store (mapper ind) (proc (get0 store0 (mapper0 ind)))))
+      (make-vector (array-rank ar)))))
 
-(define-method array-map! ((ar <array>) (proc <procedure>) ar0)
-  (array-for-each-index ar
-                        (lambda (ind)
-                          (array-set! ar ind (proc (array-ref ar0 ind))))
-                        (make-vector (array-rank ar))))
+(define-method array-map! ((ar <array-base>) (proc <procedure>) ar0 ar1)
+  (let ((set (backing-storage-setter-of (class-of ar)))
+        (store (backing-storage-of ar))
+        (mapper (mapper-of ar))
+        (get0 (backing-storage-getter-of (class-of ar0)))
+        (store0 (backing-storage-of ar0))
+        (mapper0 (mapper-of ar0))
+        (get1 (backing-storage-getter-of (class-of ar1)))
+        (store1 (backing-storage-of ar1))
+        (mapper1 (mapper-of ar1)))
+    (array-for-each-index ar0
+      (lambda (ind)
+        (set store (mapper ind) (proc (get0 store0 (mapper0 ind))
+                                      (get1 store1 (mapper1 ind)))))
+      (make-vector (array-rank ar)))))
 
-(define-method array-map! ((ar <array>) (proc <procedure>) ar0 ar1)
-  (array-for-each-index ar
-                        (lambda (ind)
-                          (array-set! ar ind
-                                      (proc (array-ref ar0 ind)
-                                            (array-ref ar1 ind))))
-                        (make-vector (array-rank ar))))
-
-(define-method array-map! ((ar <array>) (proc <procedure>) ar0 ar1 ar2)
-  (array-for-each-index ar
-                        (lambda (ind)
-                          (array-set! ar ind
-                                      (proc (array-ref ar0 ind)
-                                            (array-ref ar1 ind)
-                                            (array-ref ar2 ind))))
-                        (make-vector (array-rank ar))))
-
-(define-method array-map! ((ar <array>) (proc <procedure>) ar0 ar1 ar2 . more)
+(define-method array-map! ((ar <array-base>) (proc <procedure>) ar0 ar1 ar2 . more)
   (let1 arlist (list* ar0 ar1 ar2 more)
     (array-for-each-index ar
-                          (lambda (ind)
-                            (array-set! ar ind
-                                        (apply proc
-                                               (map (cut array-ref <> ind)
-                                                    arlist))))
-                          (make-vector (array-rank ar)))))
+      (lambda (ind)
+        (array-set! ar ind
+                    (apply proc
+                           (map (cut array-ref <> ind)
+                                arlist))))
+      (make-vector (array-rank ar)))))
 
-(define-method array-map ((sh <array>) (proc <procedure>) ar0 . more)
+(define-method array-map ((sh <array-base>) (proc <procedure>) ar0 . more)
   (apply array-map proc ar0 more))
 
 (define-method array-map ((proc <procedure>) ar0 . more)
@@ -605,17 +748,17 @@
 (define (array->vector ar)
   (with-builder (<vector> add! get :size (array-size ar))
     (array-for-each-index ar
-                          (lambda (ind)
-                            (add! (array-ref ar ind)))
-                          (make-vector (array-rank ar)))
+      (lambda (ind)
+        (add! (array-ref ar ind)))
+      (make-vector (array-rank ar)))
     (get)))
 
 (define (array->list ar)
   (with-builder (<list> add! get)
     (array-for-each-index ar
-                          (lambda (ind)
-                            (add! (array-ref ar ind)))
-                          (make-vector (array-rank ar)))
+      (lambda (ind)
+        (add! (array-ref ar ind)))
+      (make-vector (array-rank ar)))
     (get)))
 
 (provide "gauche/array")
