@@ -28,6 +28,8 @@
 (test* "#s64()" #f (s64vector? '#(0 1 2 3 4)))
 (test* "#u64()" #t (u64vector? '#u64(0 1 2 3 4)))
 (test* "#u64()" #f (u64vector? '#(0 1 2 3 4)))
+(test* "#f16()" #t (f16vector? '#f16(0.0 1.0 2.0 3.0 4.0)))
+(test* "#f16()" #f (f16vector? '#(0.0 1.0 2.0 3.0 4.0)))
 (test* "#f32()" #t (f32vector? '#f32(0.0 1.0 2.0 3.0 4.0)))
 (test* "#f32()" #f (f32vector? '#(0.0 1.0 2.0 3.0 4.0)))
 (test* "#f64()" #t (f64vector? '#f64(0.0 1.0 2.0 3.0 4.0)))
@@ -60,6 +62,9 @@
 (test* "#u64()" "#u64(0 1 2 3 4)"
        (with-output-to-string
          (lambda () (write (apply u64vector (iota 5))))))
+(test* "#f16()" "#f16()"
+       (with-output-to-string
+         (lambda () (write (f16vector)))))
 (test* "#f32()" "#f32()"
        (with-output-to-string
          (lambda () (write (f32vector)))))
@@ -109,6 +114,10 @@
        (uvrefset-tester make-u64vector u64vector-ref u64vector-set!
                         '(0 1 2 #xffffffffffffffff)
                         '#u64(0 1 2 #xffffffffffffffff)))
+(test* "f16vector-ref|set!" #t
+       (uvrefset-tester make-f16vector f16vector-ref f16vector-set!
+                        '(0.0 -1.0 1.0)
+                        '#f16(0.0 -1.0 1.0)))
 (test* "f32vector-ref|set!" #t
        (uvrefset-tester make-f32vector f32vector-ref f32vector-set!
                         '(0.0 -1.0 1.0)
@@ -222,6 +231,11 @@
                       u64vector->vector vector->u64vector
                       '#u64(0 1 #xffffffffffffffff)
                       '(0 1 #xffffffffffffffff)))
+(test* "f16vector conversion" #t
+       (uvconv-tester f16vector->list list->f16vector
+                      f16vector->vector vector->f16vector
+                      '#f16(0.0 -1.0 1.0)
+                      '(0.0 -1.0 1.0)))
 (test* "f32vector conversion" #t
        (uvconv-tester f32vector->list list->f32vector
                       f32vector->vector vector->f32vector
@@ -280,6 +294,10 @@
                       u64vector->list list->u64vector
                       '#u64(0 1 #xffffffffffffffff) #x8000000000000000))
 
+(test* "f16vector copy|fill!" #t
+       (uvcopy-tester f16vector-copy f16vector-copy! f16vector-fill!
+                      f16vector->list list->f16vector
+                      '#f16(0 -1.0 1.0) 1.0))
 (test* "f32vector copy|fill!" #t
        (uvcopy-tester f32vector-copy f32vector-copy! f32vector-fill!
                       f32vector->list list->f32vector
@@ -313,6 +331,53 @@
 (uvcopy-startend-test "uvcopy-startend u64vector"
                       u64vector u64vector-copy u64vector-fill!)
 
+(define (uvcopy!-newapi-test msg make copy!)
+  (test* #`",msg /tstart" (make 0 7 8 9)
+         (copy! (make 0 1 2 3) 1 (make 7 8 9 10)))
+  (test* #`",msg /tstart(over)" (make 0 1 2 3)
+         (copy! (make 0 1 2 3) 4 (make 7 8 9 10)))
+  (test* #`",msg /tstart,,sstart" (make 0 9 10 3)
+         (copy! (make 0 1 2 3) 1 (make 7 8 9 10) 2))
+  (test* #`",msg /tstart,,sstart,,send" (make 0 1 2 9)
+         (copy! (make 0 1 2 3) 3 (make 7 8 9 10) 2 3))
+  )
+
+(uvcopy!-newapi-test "s8vector-copy! newapi" s8vector s8vector-copy!)
+(uvcopy!-newapi-test "u8vector-copy! newapi" u8vector u8vector-copy!)
+(uvcopy!-newapi-test "s16vector-copy! newapi" s16vector s16vector-copy!)
+(uvcopy!-newapi-test "u16vector-copy! newapi" u16vector u16vector-copy!)
+(uvcopy!-newapi-test "s32vector-copy! newapi" s32vector s32vector-copy!)
+(uvcopy!-newapi-test "u32vector-copy! newapi" u32vector u32vector-copy!)
+(uvcopy!-newapi-test "s64vector-copy! newapi" s64vector s64vector-copy!)
+(uvcopy!-newapi-test "u64vector-copy! newapi" u64vector u64vector-copy!)
+(uvcopy!-newapi-test "f16vector-copy! newapi" f16vector f16vector-copy!)
+(uvcopy!-newapi-test "f32vector-copy! newapi" f32vector f32vector-copy!)
+(uvcopy!-newapi-test "f64vector-copy! newapi" f64vector f64vector-copy!)
+
+;;-------------------------------------------------------------------
+(test-section "swapping bytes")
+
+(test* "swapb s16"
+       '#s16(#x0123 #x4567 #x7654 #x3210)
+       (s16vector-swap-bytes '#s16(#x2301 #x6745 #x5476 #x1032)))
+(test* "swapb u16"
+       '#u16(#x0123 #x4567 #x7654 #x3210)
+       (u16vector-swap-bytes '#u16(#x2301 #x6745 #x5476 #x1032)))
+(test* "swapb s32"
+       '#s32(#x01234567 #x76543210)
+       (s32vector-swap-bytes '#s32(#x67452301 #x10325476)))
+(test* "swapb u32"
+       '#u32(#x01234567 #x76543210)
+       (u32vector-swap-bytes '#u32(#x67452301 #x10325476)))
+(test* "swapb s64"
+       '#s64(#x0123456789abcdef #x-123456789abcdf0)
+       (s64vector-swap-bytes
+        '#s64(#x-1032547698badcff #x1032547698badcfe)))
+(test* "swapb u64"
+       '#u64(#x0123456789abcdef #xfedcba9876543210)
+       (u64vector-swap-bytes
+        '#u64(#xefcdab8967452301 #x1032547698badcfe)))
+
 ;;-------------------------------------------------------------------
 (test-section "collection interface")
 
@@ -367,6 +432,8 @@
        (collection-tester <s64vector> (s64vector 1 2 3 4)))
 (test* "u64vector collection interface" #t
        (collection-tester <u64vector> (u64vector 1 2 3 4)))
+(test* "f16vector collection interface" #t
+       (collection-tester <f16vector> (f16vector 1 2 3 4)))
 (test* "f32vector collection interface" #t
        (collection-tester <f32vector> (f32vector 1 2 3 4)))
 (test* "f64vector collection interface" #t
@@ -592,6 +659,7 @@
          (div (make 0.0 1.0 2.0 3.0) 2.0))
   )
 
+(flonum-arith-test-generate f16)
 (flonum-arith-test-generate f32)
 (flonum-arith-test-generate f64)
 
@@ -790,6 +858,23 @@
 (dotprod-test-generate u64 #u64(1 2 3 4 18446744073709551615)
                        #u64(1 2 3 4 18446744073709551615))
 
+(dotprod-test-generate f16 #f16(0 1 2 3) #f16(4 5 6 7))
+(dotprod-test-generate f16 #f16(0 -1 2 -3) #f16(-4 5 -6 7))
+(dotprod-test-generate f16 #f16(16384 16384 16384 16384 16384)
+                       #f16(16384 16384 16384 16384 16384))
+(dotprod-test-generate f16 #f16(16384 -16384 16384 -16384 16384)
+                       #f16(16384 -16384 16384 -16384 16384))
+(dotprod-test-generate f16 #f16(32767 32767 32767 32767 32767)
+                       #f16(32767 32767 32767 32767 32767))
+(dotprod-test-generate f16 #f16(214748367 214748367 214748367 214748367 214748367)
+                       #f16(214748367 214748367 214748367 214748367 214748367))
+(dotprod-test-generate f16 #f16(9223372036854775807 1 2 3 4)
+                       #f16(9223372036854775807 1 2 3 4))
+(dotprod-test-generate f16 #f16(1 2 3 4 9223372036854775807 1)
+                       #f16(1 2 3 4 9223372036854775807 1))
+(dotprod-test-generate f16 #f16(32767 -32767 32767 -32767 32767)
+                       #f16(32767 -32767 32767 -32767 32767))
+
 (dotprod-test-generate f32 #f32(0 1 2 3) #f32(4 5 6 7))
 (dotprod-test-generate f32 #f32(0 -1 2 -3) #f32(-4 5 -6 7))
 (dotprod-test-generate f32 #f32(16384 16384 16384 16384 16384)
@@ -1143,6 +1228,28 @@
 (test* "string->u8vector (OOB)" *test-error*
        (string->u8vector "abcde" 2 6))
 
+(test* "string->u8vector!" '#u8(64 65 66 67 68)
+       (let ((v (u8vector 0 1 2 3 4)))
+         (string->u8vector! v 0 "@ABCD")))
+(test* "string->u8vector!" '#u8(64 65 66 67 68)
+       (let ((v (u8vector 0 1 2 3 4)))
+         (string->u8vector! v 0 "@ABCDEFGHIJKLMNOPQRTSUVWXYZ")))
+(test* "string->u8vector!" '#u8(64 65 66 3 4)
+       (let ((v (u8vector 0 1 2 3 4)))
+         (string->u8vector! v 0 "@AB")))
+(test* "string->u8vector!" '#u8(0 64 65 66 4)
+       (let ((v (u8vector 0 1 2 3 4)))
+         (string->u8vector! v 1 "@AB")))
+(test* "string->u8vector!" '#u8(0 1 2 3 65)
+       (let ((v (u8vector 0 1 2 3 4)))
+         (string->u8vector! v 4 "@ABCDE" 1)))
+(test* "string->u8vector!" '#u8(0 1 2 3 4)
+       (let ((v (u8vector 0 1 2 3 4)))
+         (string->u8vector! v 8 "@ABCDE" 1)))
+(test* "string->u8vector!" '#u8(0 1 2 3 4)
+       (let ((v (u8vector 0 1 2 3 4)))
+         (string->u8vector! v -1 "@ABCDE" 1)))
+
 (test* "u8vector->string" "@ABCD"
        (u8vector->string '#u8(64 65 66 67 68)))
 (test* "u8vector->string (start)" "ABCD"
@@ -1165,6 +1272,28 @@
 (test* "string->s8vector (OOB)" *test-error*
        (string->s8vector "abcde" 2 6))
 
+(test* "string->s8vector!" '#s8(64 65 66 67 68)
+       (let ((v (s8vector 0 1 2 3 4)))
+         (string->s8vector! v 0 "@ABCD")))
+(test* "string->s8vector!" '#s8(64 65 66 67 68)
+       (let ((v (s8vector 0 1 2 3 4)))
+         (string->s8vector! v 0 "@ABCDEFGHIJKLMNOPQRTSUVWXYZ")))
+(test* "string->s8vector!" '#s8(64 65 66 3 4)
+       (let ((v (s8vector 0 1 2 3 4)))
+         (string->s8vector! v 0 "@AB")))
+(test* "string->s8vector!" '#s8(0 64 65 66 4)
+       (let ((v (s8vector 0 1 2 3 4)))
+         (string->s8vector! v 1 "@AB")))
+(test* "string->s8vector!" '#s8(0 1 2 3 65)
+       (let ((v (s8vector 0 1 2 3 4)))
+         (string->s8vector! v 4 "@ABCDE" 1)))
+(test* "string->s8vector!" '#s8(0 1 2 3 4)
+       (let ((v (s8vector 0 1 2 3 4)))
+         (string->s8vector! v 8 "@ABCDE" 1)))
+(test* "string->s8vector!" '#s8(0 1 2 3 4)
+       (let ((v (s8vector 0 1 2 3 4)))
+         (string->s8vector! v -1 "@ABCDE" 1)))
+
 (test* "s8vector->string" "@ABCD"
        (s8vector->string '#s8(64 65 66 67 68)))
 (test* "s8vector->string (start)" "ABCD"
@@ -1421,80 +1550,69 @@
             '(0 1 2 3)))
 
 (test-section "array-ref")
-(test* "array-ref (list)" 'a
-       (array-ref (make-array (shape) 'a)))
-(test* "array-ref (list)" 'b
-       (array-ref (make-array (shape -1 1) 'b) -1))
-(test* "array-ref (list)" 'c
-       (array-ref (make-array (shape -1 1) 'c) 0))
-(test* "array-ref (list)" 'd
-       (array-ref (make-array (shape 1 2 3 4 5 6 7 8) 'd) 1 3 5 7))
-
-(test* "array-ref (vector)" 'a
-       (array-ref (make-array (shape) 'a) '#()))
-(test* "array-ref (vector)" 'b
-       (array-ref (make-array (shape -1 1) 'b) '#(-1)))
-(test* "array-ref (vector)" 'c
-       (array-ref (make-array (shape -1 1) 'c) '#(0)))
-(test* "array-ref (vector)" 'd
-       (array-ref (make-array (shape 1 2 3 4 5 6 7 8) 'd)
-                  '#(1 3 5 7)))
-
-(test* "array-ref (array)" 'a
-       (array-ref (make-array (shape) 'a)
-                  (array (shape 0 0))))
-(test* "array-ref (array)" 'b
-       (array-ref (make-array (shape -1 1) 'b)
-                  (array (shape 0 1) -1)))
-(test* "array-ref (array)" 'c
-       (array-ref (make-array (shape -1 1) 'c)
-                  (array (shape 0 1) 0)))
-(test* "array-ref (array)" 'd
-       (array-ref (make-array (shape 1 2 3 4 5 6 7 8) 'd)
-                  (array (shape 0 4) 1 3 5 7)))
+(for-each
+ (lambda (ls)
+   (let ((a (car ls)))
+     (for-each
+      (lambda (t)
+        (test* (format "array-ref ~S:" (cdr t)) (car t)
+          (apply array-ref a (cdr t))))
+      (cdr ls))))
+ '((#,(<array> () a) (a))
+   (#,(<array> (-1 1) a b) (a -1) (b 0))
+   (#,(<array> (1 2 3 4 5 6 7 8) a) (a 1 3 5 7))
+   (#,(<array> (1 3 4 6 7 9 10 12) a b c d e f g h i j k l m n o p)
+    (a 1 4 7 10) (b 1 4 7 11) (i 2 4 7 10) (n 2 5 7 11))
+   (#,(<array> () a) (a #()))
+   (#,(<array> (-1 1) a b) (a #(-1)) (b #(0)))
+   (#,(<array> (1 2 3 4 5 6 7 8) a) (a #(1 3 5 7)))
+   (#,(<array> (1 3 4 6 7 9 10 12) a b c d e f g h i j k l m n o p)
+    (a #(1 4 7 10)) (b #(1 4 7 11)) (i #(2 4 7 10)) (n #(2 5 7 11)))
+   (#,(<array> () a) (a #,(<array> (0 0))))
+   (#,(<array> (-1 1) a b) (a #,(<array> (0 1) -1)) (b #,(<array> (0 1) 0)))
+   (#,(<array> (1 2 3 4 5 6 7 8) a) (a #,(<array> (0 4) 1 3 5 7)))
+   (#,(<array> (1 3 4 6 7 9 10 12) a b c d e f g h i j k l m n o p)
+    (a #,(<array> (0 4) 1 4 7 10)) (b #,(<array> (0 4) 1 4 7 11))
+    (i #,(<array> (0 4) 2 4 7 10)) (n #,(<array> (0 4) 2 5 7 11)))
+   (#,(<u8array> () 2) (2))
+   (#,(<u8array> (-1 1) 2 3) (2 -1) (3 0))
+   (#,(<u8array> (1 2 3 4 5 6 7 8) 2) (2 1 3 5 7))
+   (#,(<u8array> (1 3 4 6 7 9 10 12) 2 3 5 7 11 13 17 23 29 31 37 39 41 43 47 51)
+    (2 1 4 7 10) (3 1 4 7 11) (29 2 4 7 10) (43 2 5 7 11))
+   ))
 
 (test-section "array-set!")
-(test* "array-set! (list)" 'a
-       (let ((arr (make-array (shape) 'o)))
-         (array-set! arr 'a)
-         (array-ref arr)))
-(test* "array-set! (list)" '(b c)
-       (let ((arr (make-array (shape -1 1) 'o)))
-         (array-set! arr -1 'b)
-         (array-set! arr 0 'c)
-         (list (array-ref arr -1) (array-ref arr 0))))
-(test* "array-set! (list)" 'd
-       (let ((arr (make-array (shape 1 2 3 4 5 6 7 8) 'o)))
-         (array-set! arr 1 3 5 7 'd)
-         (array-ref arr 1 3 5 7)))
-
-(test* "array-set! (vector)" 'a
-       (let ((arr (make-array (shape) 'o)))
-         (array-set! arr '#() 'a)
-         (array-ref arr)))
-(test* "array-set! (vector)" '(b c)
-       (let ((arr (make-array (shape -1 1) 'o)))
-         (array-set! arr '#(-1) 'b)
-         (array-set! arr '#(0) 'c)
-         (list (array-ref arr -1) (array-ref arr 0))))
-(test* "array-set! (vector)" 'd
-       (let ((arr (make-array (shape 1 2 3 4 5 6 7 8) 'o)))
-         (array-set! arr '#(1 3 5 7) 'd)
-         (array-ref arr 1 3 5 7)))
-
-(test* "array-set! (array)" 'a
-       (let ((arr (make-array (shape) 'o)))
-         (array-set! arr 'a)
-         (array-ref arr)))
-(test* "array-set! (array)" '(b c)
-       (let ((arr (make-array (shape -1 1) 'o)))
-         (array-set! arr (array (shape 0 1) -1) 'b)
-         (array-set! arr (array (shape 0 1) 0) 'c)
-         (list (array-ref arr -1)  (array-ref arr 0))))
-(test* "array-set! (array)" 'd
-       (let ((arr (make-array (shape 1 2 3 4 5 6 7 8) 'o)))
-         (array-set! arr (array (shape 0 4) 1 3 5 7) 'd)
-         (array-ref arr 1 3 5 7)))
+(for-each
+ (lambda (ls)
+   (let ((a (car ls)))
+     (for-each
+      (lambda (t)
+        (test* (format "array-set! ~S:" (cdr t)) (car t)
+          (begin (apply array-set! a (append (cdr t) (list (car t))))
+                 (apply array-ref a (cdr t)))))
+      (cdr ls))))
+ '((#,(<array> () a) (x))
+   (#,(<array> (-1 1) a b) (y -1) (z 0))
+   (#,(<array> (1 2 3 4 5 6 7 8) a) (x 1 3 5 7))
+   (#,(<array> (1 3 4 6 7 9 10 12) a b c d e f g h i j k l m n o p)
+    (w 1 4 7 10) (x 1 4 7 11) (y 2 4 7 10) (z 2 5 7 11))
+   (#,(<array> () a) (x #()))
+   (#,(<array> (-1 1) a b) (x #(-1)) (y #(0)))
+   (#,(<array> (1 2 3 4 5 6 7 8) a) (x #(1 3 5 7)))
+   (#,(<array> (1 3 4 6 7 9 10 12) a b c d e f g h i j k l m n o p)
+    (w #(1 4 7 10)) (x #(1 4 7 11)) (y #(2 4 7 10)) (z #(2 5 7 11)))
+   (#,(<array> () a) (x #,(<array> (0 0))))
+   (#,(<array> (-1 1) a b) (x #,(<array> (0 1) -1)) (y #,(<array> (0 1) 0)))
+   (#,(<array> (1 2 3 4 5 6 7 8) a) (x #,(<array> (0 4) 1 3 5 7)))
+   (#,(<array> (1 3 4 6 7 9 10 12) a b c d e f g h i j k l m n o p)
+    (w #,(<array> (0 4) 1 4 7 10)) (x #,(<array> (0 4) 1 4 7 11))
+    (y #,(<array> (0 4) 2 4 7 10)) (z #,(<array> (0 4) 2 5 7 11)))
+   (#,(<u8array> () 2) (102))
+   (#,(<u8array> (-1 1) 2 3) (102 -1) (103 0))
+   (#,(<u8array> (1 2 3 4 5 6 7 8) 2) (102 1 3 5 7))
+   (#,(<u8array> (1 3 4 6 7 9 10 12) 2 3 5 7 11 13 17 23 29 31 37 39 41 43 47 51)
+    (102 1 4 7 10) (103 1 4 7 11) (129 2 4 7 10) (143 2 5 7 11))
+   ))
 
 (test* "array-valid-index? (list)" #t
        (array-valid-index? (make-array (shape) 'o)))
@@ -1533,7 +1651,7 @@
 (test* "array-valid-index? (array)" #f
        (array-valid-index? (make-array (shape -1 1) 'o) #,(<array> (0 1) 1)))
 (test* "array-valid-index? (array)" #f
-       (array-valid-index? (make-array (shape 1 2 3 4 5 6 7 8) 'o) #,(<array> (0 4) 1 4 5 7)))
+      (array-valid-index? (make-array (shape 1 2 3 4 5 6 7 8) 'o) #,(<array> (0 4) 1 4 5 7)))
 
 ;;; Share and change:
 ;;;
@@ -1907,6 +2025,12 @@
     (array-retabulate! ar (lambda (v) (/ 1.0 (array-ref ar v))) vec)
     ar))
 
+(test* "array-retabulate! (uniform)" #,(<f64array> (0 2 0 2) 1.0 0.5 0.25 0.125)
+  (let ((ar #,(<f64array> (0 2 0 2) 1 2 4 8))
+        (vec (make-vector 2)))
+    (array-retabulate! ar (lambda (v) (/ 1.0 (array-ref ar v))) vec)
+    ar))
+
 (test* "array-map-1" #,(<array> (0 2 0 2) 1 4 9 16)
   (array-map (lambda (x) (* x x)) #,(<array> (0 2 0 2) 1 2 3 4)))
 
@@ -1928,6 +2052,13 @@
              #,(<array> (0 2 0 2) .1 .2 .3 .4)
              #,(<array> (0 2 0 2) 10.0 1.0 0.1 0.01)))
 
+(test* "array-map-uniform" #,(<array> (0 2 0 2) 111.0 22.2 3.33 .444)
+  (array-map (lambda (a b c d) (/ (truncate (* (* d (+ a b c)) 10000)) 10000))
+             #,(<u8array> (0 2 0 2) 1 2 3 4)
+             #,(<s32array> (0 2 0 2) 10 20 30 40)
+             #,(<f32array> (0 2 0 2) .1 .2 .3 .4)
+             #,(<f64array> (0 2 0 2) 10.0 1.0 0.1 0.01)))
+
 (test* "array-every-1" #t
   (array-every even? #,(<array> (0 0))))
 (test* "array-every-2" #t
@@ -1950,6 +2081,79 @@
 (test* "array-any-5" #f
   (array-any even? #,(<array> (0 2 0 2) 1 3 5 7)))
 
+(test-section "matrices")
+
+;; zero-base the shape
+(define (array-normalize a)
+  (define (every-other pred ls)
+    (if (null? ls) #t (and (pred (car ls)) (every-other pred (cddr ls)))))
+  (if (every-other zero? (array->list (array-shape a)))
+    a
+    (subarray a (array-shape a))))
+
+(define (approx-equal? x y . opt)
+  (< (abs (- x y)) (get-optional opt 0.0000001)))
+
+(define (array-approx-equal? a b)
+  (or (eq? a b) (array-equal? a b approx-equal?)))
+
+(let ((i 0))
+  (for-each
+   (lambda (t)
+     (let-optionals* t (ar (inv #f) (det 0))
+       (test* (format "array-inverse-~D" (inc! i)) inv
+         (array-inverse ar)
+         array-approx-equal?)
+       (when inv
+         (test* (format "array-inverse-~D" (inc! i)) (array-normalize ar)
+           (array-inverse inv)
+           array-approx-equal?))
+       (test* (format "determinant-~D" (inc! i)) det
+         (determinant ar)
+         approx-equal?)))
+   '((#,(<array> (0 2 0 2) 1 2 3 4)
+      #,(<array> (0 2 0 2) -2.0 1.0 1.5 -0.5)
+      -2)
+     (#,(<array> (1 3 1 3) 1 2 3 4)
+      #,(<array> (0 2 0 2) -2.0 1.0 1.5 -0.5)
+      -2)
+     (#,(<array> (3 5 7 9) 1 2 3 4)
+      #,(<array> (0 2 0 2) -2.0 1.0 1.5 -0.5)
+      -2)
+     (#,(<array> (0 3 0 3) 1 5 2 1 1 7 0 -3 4)
+      #,(<array> (0 3 0 3) -25 26 -33 4 -4 5 3 -3 4)
+      -1)
+     (#,(<array> (0 3 0 3) 2 0 1 1 1 0 3 2 1)
+      #,(<array> (0 3 0 3) 1 2 -1 -1 -1 1 -1 -4 2)
+      1)
+     (#,(<array> (0 3 0 3) 1 -1 3 2 1 2 -2 -2 1)
+      #,(<array> (0 3 0 3) 1 -1 -1 -1.2 1.4 0.8 -0.4 0.8 0.6)
+      5)
+     (#,(<array> (0 2 0 2) 1 2 3 6))
+     )))
+
+(let ((i 0))
+  (for-each
+   (lambda (t)
+     (let-optionals* t (a b c)
+       (test* (format "array-mul-~D" (inc! i)) c
+         (array-mul a b)
+         array-approx-equal?)))
+   '((#,(<array> (0 2 0 2) 1 2 3 4)
+      #,(<array> (0 2 0 2) 4 3 2 1)
+      #,(<array> (0 2 0 2) 8 5 20 13))
+     (#,(<array> (1 3 1 3) 1 1 1 1)
+      #,(<array> (2 4 2 4) 2 2 2 2)
+      #,(<array> (0 2 0 2) 4 4 4 4))
+     (#,(<u8array> (3 5 7 9) 3 1 4 1)
+      #,(<u8array> (55 57 17 19) 2 7 8 1)
+      #,(<u8array> (0 2 0 2) 14 22 16 29))
+     (#,(<s16array> (3 4 1 9) 1 -2 3 -4 5 -6 7 -8)
+      #,(<s16array> (3 11 5 6) 1 -2 3 -4 5 -6 7 -8)
+      #,(<s16array> (0 1 0 1) 204))
+     )))
+
+
 ;;-------------------------------------------------------------------
 ;; NB: copy-port uses read-block! and write-block for block copy,
 ;;     so we test it here.
