@@ -1,7 +1,7 @@
 /*
  * uvinit.c - initialize routine for uvector extension
  *
- *   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,11 +30,12 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: uvinit.c,v 1.11 2006/11/04 09:56:59 shirok Exp $
+ *  $Id: uvinit.c,v 1.18 2007/09/15 07:23:39 shirok Exp $
  */
 
 #include <gauche.h>
 #include <gauche/extend.h>
+#define EXTUVECTOR_EXPORTS
 #include "gauche/uvector.h"
 #include "uvectorP.h"
 
@@ -58,6 +59,7 @@ static ScmObj read_uvector(ScmPort *port, const char *tag,
     else if (strcmp(tag, "u32") == 0) uv = Scm_ListToU32Vector(list, 0);
     else if (strcmp(tag, "s64") == 0) uv = Scm_ListToS64Vector(list, 0);
     else if (strcmp(tag, "u64") == 0) uv = Scm_ListToU64Vector(list, 0);
+    else if (strcmp(tag, "f16") == 0) uv = Scm_ListToF16Vector(list, 0);
     else if (strcmp(tag, "f32") == 0) uv = Scm_ListToF32Vector(list, 0);
     else if (strcmp(tag, "f64") == 0) uv = Scm_ListToF64Vector(list, 0);
     else Scm_Error("invalid unform vector tag: %s", tag);
@@ -73,10 +75,8 @@ static ScmObj read_uvector(ScmPort *port, const char *tag,
  */
 extern void Scm_Init_uvlib(ScmModule *);
 extern void Scm_Init_uvseq(void);
-SCM_EXTERN ScmObj (*Scm_ReadUvectorHook)(ScmPort *port, const char *tag,
-                                         ScmReadContext *ctx);
  
-void Scm_Init_libgauche_uvector(void)
+SCM_EXTENSION_ENTRY void Scm_Init_libgauche_uvector(void)
 {
     ScmModule *m;
 
@@ -91,10 +91,11 @@ void Scm_Init_libgauche_uvector(void)
     Scm_InitStaticClassWithMeta(&Scm_U32VectorClass, "<u32vector>", m, NULL, SCM_NIL, NULL, 0);
     Scm_InitStaticClassWithMeta(&Scm_S64VectorClass, "<s64vector>", m, NULL, SCM_NIL, NULL, 0);
     Scm_InitStaticClassWithMeta(&Scm_U64VectorClass, "<u64vector>", m, NULL, SCM_NIL, NULL, 0);
+    Scm_InitStaticClassWithMeta(&Scm_F16VectorClass, "<f16vector>", m, NULL, SCM_NIL, NULL, 0);
     Scm_InitStaticClassWithMeta(&Scm_F32VectorClass, "<f32vector>", m, NULL, SCM_NIL, NULL, 0);
     Scm_InitStaticClassWithMeta(&Scm_F64VectorClass, "<f64vector>", m, NULL, SCM_NIL, NULL, 0);
 
     Scm_Init_uvlib(m);
     Scm_Init_uvseq();
-    Scm_ReadUvectorHook = read_uvector;
+    Scm__InstallReadUvectorHook(read_uvector);
 }
