@@ -1,7 +1,7 @@
 ;;;
 ;;; module related utility functions.  to be autoloaded.
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: modutil.scm,v 1.5 2005/05/22 03:27:32 shirok Exp $
+;;;  $Id: modutil.scm,v 1.7 2007/03/02 07:39:08 shirok Exp $
 ;;;
 
 (define-module gauche.modutil
@@ -50,7 +50,7 @@
         (cond ((null? syms) (reverse! r))
               ((not (symbol? (car syms)))
                (error "non-symbol in export-if-defined form:" (car syms)))
-              ((symbol-bound? (car syms))
+              ((global-variable-bound? #f (car syms))
                (loop (cdr syms) (cons (car syms) r)))
               (else (loop (cdr syms) r))))))
 
