@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: numerical.scm,v 1.21 2007/03/11 21:01:29 shirok Exp $
+;;;  $Id: numerical.scm,v 1.22 2007/12/25 12:00:42 shirok Exp $
 ;;;
 
 (select-module gauche)
@@ -97,85 +97,85 @@
 ;;  Cf. Teiji Takagi: "Kaiseki Gairon" pp.193--198
 
 (define-in-module scheme (exp z)
-  (cond ((real? z) (%exp z))
-        ((complex? z)
-         (receive (x y) (%complex->real/imag z)
-           (make-polar (%exp x) y)))
-        (else (error "number required, but got" z))))
+  (cond [(real? z) (%exp z)]
+        [(complex? z) (make-polar (%exp (real-part z)) (imag-part z))]
+        [else (error "number required, but got" z)]))
 
 (define-in-module scheme (log z)
-  (cond ((real? z) (%log z))
-        ((complex? z)
-         (make-rectangular (%log (magnitude z)) (angle z)))
-        (else (error "number required, but got" z))))
+  (cond [(real? z) (%log z)]
+        [(complex? z) (make-rectangular (%log (magnitude z)) (angle z))]
+        [else (error "number required, but got" z)]))
 
 (define-in-module scheme (sqrt z)
-  (cond ((real? z) (%sqrt z))
-        ((complex? z) 
-         (make-polar (%sqrt (magnitude z)) (/ (angle z) 2.0)))
-        (else (error "number required, but got" z))))
+  (cond [(real? z) (%sqrt z)]
+        [(complex? z) (make-polar (%sqrt (magnitude z)) (/ (angle z) 2.0))]
+        [else (error "number required, but got" z)]))
 
 (define-in-module scheme (expt x y)
-  (cond ((real? x)
-         (cond ((real? y) (%expt x y))
-               ((number? y)
-                (receive (real imag) (%complex->real/imag y)
-                  (* (%expt x real) (exp (* +i imag (%log x))))))
-               (else (error "number required, but got" y))))
-        ((number? x) (exp (* y (log x))))
-        (else (error "number required, but got" x))))
+  (cond [(real? x)
+         (cond [(real? y) (%expt x y)]
+               [(number? y)
+                (* (%expt x (real-part y))
+                   (exp (* +i (imag-part y) (%log x))))]
+               [else (error "number required, but got" y)])]
+        [(number? x) (exp (* y (log x)))]
+        [else (error "number required, but got" x)]))
 
 (define-in-module scheme (cos z)
-  (cond ((real? z) (%cos z))
-        ((number? z)
-         (receive (x y) (%complex->real/imag z)
+  (cond [(real? z) (%cos z)]
+        [(number? z)
+         (let ((x (real-part z))
+               (y (imag-part z)))
            (make-rectangular (* (%cos x) (%cosh y))
-                             (- (* (%sin x) (%sinh y))))))
-        (else (error "number required, but got" z))))
+                             (- (* (%sin x) (%sinh y)))))]
+        [else (error "number required, but got" z)]))
 
 (define (cosh z)
-  (cond ((real? z) (%cosh z))
-        ((number? z)
-         (receive (x y) (%complex->real/imag z)
+  (cond [(real? z) (%cosh z)]
+        [(number? z)
+         (let ((x (real-part z))
+               (y (imag-part z)))
            (make-rectangular (* (%cosh x) (%cos y))
-                             (* (%sinh x) (%sin y)))))
-        (else (error "number required, but got" z))))
+                             (* (%sinh x) (%sin y))))]
+        [else (error "number required, but got" z)]))
 
 (define-in-module scheme (sin z)
-  (cond ((real? z) (%sin z))
-        ((number? z)
-         (receive (x y) (%complex->real/imag z)
+  (cond [(real? z) (%sin z)]
+        [(number? z)
+         (let ((x (real-part z))
+               (y (imag-part z)))
            (make-rectangular (* (%sin x) (%cosh y))
-                             (* (%cos x) (%sinh y)))))
-        (else (error "number required, but got" z))))
+                             (* (%cos x) (%sinh y))))]
+        [else (error "number required, but got" z)]))
 
 (define (sinh z)
-  (cond ((real? z) (%sinh z))
-        ((number? z)
-         (receive (x y) (%complex->real/imag z)
+  (cond [(real? z) (%sinh z)]
+        [(number? z)
+         (let ((x (real-part z))
+               (y (imag-part z)))
            (make-rectangular (* (%sinh x) (%cos y))
-                             (* (%cosh x) (%sin y)))))
-        (else (error "number required, but got" z))))
+                             (* (%cosh x) (%sin y))))]
+        [else (error "number required, but got" z)]))
 
 (define-in-module scheme (tan z)
-  (cond ((real? z) (%tan z))
-        ((number? z)
+  (cond [(real? z) (%tan z)]
+        [(number? z)
          (let ((iz (* +i z)))
            (* -i
               (/ (- (exp iz) (exp (- iz)))
-                 (+ (exp iz) (exp (- iz)))))))
-        (else (error "number required, but got" z))))
+                 (+ (exp iz) (exp (- iz))))))]
+        [else (error "number required, but got" z)]))
 
 (define (tanh z)
-  (cond ((real? z) (%tanh z))
-        ((number? z)
+  (cond [(real? z) (%tanh z)]
+        [(number? z)
          (/ (- (exp z) (exp (- z)))
-            (+ (exp z) (exp (- z)))))
-        (else (error "number required, but got" z))))
+            (+ (exp z) (exp (- z))))]
+        [else (error "number required, but got" z)]))
 
 (define-in-module scheme (asin z)
-  (cond ((real? z) (%asin z))
-        ((number? z)
+  (cond [(real? z) (%asin z)]
+        [(number? z)
          ;; The definition of asin is
          ;;   (* -i (log (+ (* +i z) (sqrt (- 1 (* z z))))))
          ;; This becomes unstable when the term in the log is reaching
@@ -183,27 +183,29 @@
          ;; gets closer to zero when |z| gets bigger, but for large |z|,
          ;; k is prone to lose precision and starts drifting around
          ;; the point zero.
-         ;; For now, I let asin to return NaN for large z's.
-         (if (> (magnitude z) 1.0e5)
-             (make-rectangular (log 0.0) (log 0.0)) ;NaN+NaNi
-             (* -i (log (+ (* +i z) (sqrt (- 1 (* z z))))))))
-        (else (error "number required, but got" z))))
+         ;; For now, I let asin to return NaN in such cases.
+         (let1 zz (+ (* +i z) (sqrt (- 1 (* z z))))
+           (if (< (/. (magnitude zz) (magnitude z)) 1.0e-8)
+             (make-rectangular +nan.0 +nan.0)
+             (* -i (log zz))))]
+        [else (error "number required, but got" z)]))
 
 (define (asinh z)
-  (if (> (magnitude z) 1.0e5)
-      (make-rectangular (log 0.0) (log 0.0)) ;NaN+NaNi
-      (log (+ z (sqrt (+ (* z z) 1))))))
+  (let1 zz (+ z (sqrt (+ (* z z) 1)))
+    (if (< (/. (magnitude zz) (magnitude z)) 1.0e-8)
+      (make-rectangular +nan.0 +nan.0)
+      (log (+ z (sqrt (+ (* z z) 1)))))))
 
 (define-in-module scheme (acos z)
-  (cond ((real? z) (%acos z))
-        ((number? z)
+  (cond [(real? z) (%acos z)]
+        [(number? z)
          ;; The definition of acos is
          ;;  (* -i (log (+ z (* +i (sqrt (- 1 (* z z)))))))))
          ;; This also falls in the victim of numerical unstability; worse than
          ;; asin, sometimes the real part of marginal value "hops" between
          ;; +pi and -pi.  It's rather stable to use asin.
-         (- 1.5707963267948966 (asin z)))
-        (else (error "number required, but got" z))))
+         (- 1.5707963267948966 (asin z))]
+        [else (error "number required, but got" z)]))
 
 (define (acosh z)
   ;; See the discussion of CLtL2, pp. 313-314
@@ -212,14 +214,14 @@
 
 (define-in-module scheme (atan z . x)
   (if (null? x)
-      (cond ((real? z) (%atan z))
-            ((number? z)
-             (let ((iz (* z +i)))
-               (/ (- (log (+ 1 iz))
-                     (log (- 1 iz)))
-                  +2i)))
-            (else (error "number required, but got" z)))
-      (%atan z (car x))))
+    (cond [(real? z) (%atan z)]
+          [(number? z)
+           (let1 iz (* z +i)
+             (/ (- (log (+ 1 iz))
+                   (log (- 1 iz)))
+                +2i))]
+          [else (error "number required, but got" z)])
+    (%atan z (car x))))
 
 (define (atanh z)
   (/ (- (log (+ 1 z)) (log (- 1 z))) 2))
