@@ -30,28 +30,31 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: 822.scm,v 1.20 2007/03/02 07:39:10 shirok Exp $
+;;;  $Id: 822.scm,v 1.22 2007/12/24 05:31:32 shirok Exp $
 ;;;
 
 ;; Parser and constructor of the message defined in
-;; RFC2822 Internet Message Format <ftp://ftp.isi.edu/in-notes/rfc2822.txt>
+;; RFC2822 Internet Message Format
+;;         http://www.ietf.org/rfc/rfc2822.txt
 
 (define-module rfc.822
   (use srfi-1)
-  (use srfi-2)
   (use srfi-13)
   (use srfi-19)
   (use text.parse)
   (use gauche.regexp)
-  (export rfc822-header->list rfc822-header-ref
+  (use util.match)
+  (export <rfc822-parse-error> rfc822-parse-errorf
+          rfc822-read-headers rfc822-header->list rfc822-header-ref
           rfc822-skip-cfws
           *rfc822-atext-chars* *rfc822-standard-tokenizers*
           rfc822-atom rfc822-dot-atom rfc822-quoted-string
           rfc822-next-token rfc822-field->tokens
           rfc822-parse-date rfc822-date->date
+
+          rfc822-invalid-header-field rfc822-write-headers
           )
   )
-
 (select-module rfc.822)
 
 ;;=================================================================
@@ -59,9 +62,35 @@
 ;;
 
 ;;-----------------------------------------------------------------
-;; Generic header parser, recognizes folded line and field names
+;; A condition reported when rfc822 parser finds an error.
 ;;
-(define (rfc822-header->list iport . args)
+;; For most practical applications, it is useful to handle
+;; rfc822 syntax errors gracefully.  So the rfc822 and mime procedures
+;; tries to return a reasonable value even if the input is not strictly
+;; conforming the RFCs.  Some routines have an option to be more strict,
+;; and they raise <rfc822-parse-error> when the option is given and
+;; they find a violation.
+;;
+;; TODO: We need some consistent way to switch these 'graceful' and
+;; 'strict' modes throughout the libraries; otherwise the API would be
+;; pretty confusing.  Something to consider as we add more high-level
+;; parsers.
+
+(define-condition-type <rfc822-parse-error> <error> #f
+  (header-field-name)
+  (header-field-body))
+
+(define (rfc822-parse-errorf name body fmt . args)
+  (apply errorf <rfc822-parse-error>
+         :header-field-name name
+         :header-field-body body
+         fmt args))
+
+;;-----------------------------------------------------------------
+;; Generic header parser.  Returns ((name body) ...)
+;; Does process unfolding.
+;; May throw <rfc822-parse-error> if :strict? is true.
+(define (rfc822-read-headers iport . args)
   (let-keywords args ((strict? #f)
                       (reader (cut read-line <> #t)))
 
@@ -76,41 +105,44 @@
     (let loop ((r '())
                (line (reader iport)))
       (cond
-       ((eof-object? line) (reverse! r))
-       ((string-null? line) (reverse! r))
-       (else
+       [(eof-object? line) (reverse! r)]
+       [(string-null? line) (reverse! r)]
+       [else
         (receive (n body) (string-scan line #\: 'both)
           (let1 name (and-let* (((string? n))
                                  (name (string-incomplete->complete n))
                                  (name (string-trim-both name))
                                  ((string-every #[\x21-\x39\x3b-\x7e] name)))
                         (string-downcase name))
-            (if name
+            (cond
+             [name
               (let loop2 ((nline (reader iport))
                           (bodies (list (drop-leading-fws body))))
-                (cond ((eof-object? nline)
+                (cond [(eof-object? nline)
                        ;; maybe premature end of the message
                        (if strict?
-                         (error "premature end of message header")
-                         (reverse! (accum name bodies r))))
-                      ((string-null? nline)     ;; end of the header
-                       (reverse! (accum name bodies r)))
-                      ((memv (string-byte-ref nline 0) '(9 32))
+                         (rfc822-parse-errorf
+                          #f #f "premature end of message header")
+                         (reverse! (accum name bodies r)))]
+                      [(string-null? nline)     ;; end of the header
+                       (reverse! (accum name bodies r))]
+                      [(memv (string-byte-ref nline 0) '(9 32))
                        ;; careful for byte strings
-                       (loop2 (reader iport) (cons nline bodies)))
-                      (else
-                       (loop (accum name bodies r) nline)))
-                )
-              (if strict?
-                (error "bad header line:" line)
-                (loop r (reader iport)))))))
-       ))
+                       (loop2 (reader iport) (cons nline bodies))]
+                      [else
+                       (loop (accum name bodies r) nline)]))]
+             [strict? (rfc822-parse-errorf #f #f "bad header line: ~s" line)]
+             [else (loop r (reader iport))])))])
+      )
     ))
 
 (define (rfc822-header-ref header field-name . maybe-default)
   (cond ((assoc field-name header) => cadr)
         (else (get-optional maybe-default #f))))
 
+;; backward compatibility
+(define rfc822-header->list rfc822-read-headers)
+
 ;;------------------------------------------------------------------
 ;; Comments, quoted pairs, atoms and quoted string.  Section 3.2
 ;;
@@ -246,5 +278,55 @@
 
 ;; to be written
 
+;;=================================================================
+;; Constructors
+;;
 
+;; Writes out the header fields specified by HEADERS, which is
+;; ((name body) ...).
+(define (rfc822-write-headers headers . keys)
+  (let-keywords keys ((output (current-output-port))
+                      (check :error) ; #f, :ignore, :error or <procedure>
+                      (continue #f))
+    (define (process headers)
+      (dolist (field headers)
+        (display (car field) output)
+        (display ": " output)
+        (display (cadr field) output)
+        (unless (string-suffix? "\r\n" (cadr field))
+          (display "\r\n" output)))
+      (unless continue (display "\r\n" output)))
+    (define (bad name body reason)
+      (errorf "Illegal RFC2822 header field data (~a): ~a: ~,,,,80:a" reason name body))
+    (if (memv check '(#f :ignore))
+      (process headers)
+      (let loop ((hs headers)
+                 (hs2 '()))
+        (match hs
+          [() (process (reverse hs2))]
+          [((name body) . rest)
+           (cond [(rfc822-invalid-header-field (string-append name ": " body))
+                  => (lambda (reason)
+                       (if (eq? check :error)
+                         (bad name body reason)
+                         (receive (name2 body2) (check name body reason)
+                           (if (and (equal? name name2) (equal? body body2))
+                             (bad name body reason)
+                             (loop `((,name2 ,body2) . ,rest) hs2)))))]
+                 [else (loop rest `((,name ,body) . ,hs2))])]
+          [else
+           (error "Invalid header data:" headers)])))
+    ))
+
+(define (rfc822-invalid-header-field body)
+  (cond [(string-incomplete? body) 'incomplete-string]
+        [(string-index body #[^\x01-\x7f]) 'bad-character]
+        [else
+         (let1 lines (string-split body "\r\n ")
+           (cond [(any (lambda (s) (> (string-size s) 998)) lines)
+                  'line-too-long]
+                 [(any (lambda (s) (string-index s #[\x0d\x0a])) lines)
+                  'stray-crlf]
+                 [else #f]))]))
+           
 (provide "rfc/822")
