@@ -1,7 +1,7 @@
 ;;;
 ;;; html-lite.scm - lightweight HTML construction
 ;;;  
-;;;   Copyright (c) 2000-2003 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: html-lite.scm,v 1.14 2004/05/21 11:10:11 shirok Exp $
+;;;  $Id: html-lite.scm,v 1.18 2007/03/02 07:39:11 shirok Exp $
 ;;;
 
 (define-module text.html-lite
@@ -91,7 +91,7 @@
     ))
 
 (define (html-doctype . args)
-  (let-keywords* args ((type :html-4.01-strict))
+  (let-keywords args ((type :html-4.01-strict))
     (cond ((find (lambda (e) (memq type (car e))) *doctype-alist*)
            => caddr )
           (else (error "Unknown doctype type spec" type)))))
@@ -119,14 +119,14 @@
             (else (values (reverse attrs) args))))
 
     (if empty?
-        (lambda args
-          (receive (attr args) (get-attr args '())
-            (unless (null? args)
-              (error "element ~s can't have content: ~s" args))
-            (list "<" name attr " />")))
-        (lambda args
-          (receive (attr args) (get-attr args '())
-            (list "<" name attr ">" args "</" name "\n>"))))))
+      (lambda args
+        (receive (attr args) (get-attr args '())
+          (unless (null? args)
+            (errorf "element ~s can't have content: ~s" name args))
+          (list "<" name attr " />")))
+      (lambda args
+        (receive (attr args) (get-attr args '())
+          (list "<" name attr ">" args "</" name "\n>"))))))
 
 (define-macro (define-html-elements . elements)
   (define (make-scheme-name name)
