@@ -1,7 +1,7 @@
 ;;;
 ;;; text.sql - SQL parsing
 ;;;  
-;;;   Copyright (c) 2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2005-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: sql.scm,v 1.1 2005/08/31 05:46:32 shirok Exp $
+;;;  $Id: sql.scm,v 1.4 2007/03/02 07:39:11 shirok Exp $
 ;;;
 
 ;; *EXPERIMENTAL*
@@ -100,11 +100,12 @@
 ;;; Tokenizer
 ;;;   Returns a list of tokens.  Each token is either one of the
 ;;;   following form.
-;;;   
-;;;    <symbol>              Regular identifier, or special delimiter:
-;;;                          + - * / < = > <> <= >= ||
-;;;    <character>           Special delimiter: 
-;;;                          #\, #\. #\( #\) #\;
+;;;
+;;;    <string>              Regular identifiers.
+;;;    <symbol>              Special delimiters:
+;;;                           + - * / < = > <> <= >= ||
+;;;    <char>                Special delimiters:
+;;;                           , . ( ) ;
 ;;;    (delimited <string>)  Delimited identifier
 ;;;    (parameter <num>)     Positional parameter (?)
 ;;;    (parameter <string>)  Named parameter (:foo)
@@ -113,6 +114,9 @@
 ;;;    (bitstring <string>)  Binary string.  <string> is like "01101"
 ;;;    (hexstring <string>)  Binary string.  <string> is like "3AD20"
 ;;;
+;;;   NB: The difference of two kinds of special delimiters is that
+;;;       the former is sometimes written with surrounding
+;;;       whitespaces, while the latter is almost never written with them.
 
 (define (sql-tokenize sql-string)
   (define parameter-count 0) ;; positional parameter count
@@ -159,29 +163,36 @@
                  (entry (m 'after) (cons `(parameter ,(m 1)) r))))
            ((#/^\w+/ s)
             => (lambda (m)
-                 (entry (m 'after)
-                        (cons (string->symbol (string-downcase (m))) r))))
+                 (entry (m 'after) (cons (m) r))))
            (else (e "invalid SQL token beginning with ~s in: ~s"
                     c sql-string)))))))
   ;;
   ;; subscanners
   ;;
+  (define (scan-quote s r q succ efmt)
+    (let loop ((pos 0))
+      (or (and-let* ((pos (string-index s q pos)))
+            (if (eqv? (string-ref s (+ pos 1) #f) q)
+              (loop (+ pos 2))
+              (succ (string-take s pos) (string-drop s (+ pos 1)))))
+          (e efmt sql-string))))
+  
   (define (scan-string s r)
-    (cond ((#/^'((?:[^']|'')*)'/ s)
-           => (lambda (m)
-                (entry (m 'after)
-                       (cons `(string ,(regexp-replace-all #/''/ (m 1) "'"))
-                             r))))
-          (else
-           (e "unterminated string literal in SQL: ~s" sql-string))))
+    (scan-quote (string-drop s 1) r #\'
+                (lambda (body rest)
+                  (entry rest
+                         `((string ,(regexp-replace-all #/''/ body "'"))
+                           . ,r)))
+                "unterminated string literal in SQL: ~s"))
+
   (define (scan-delimited s r)
-    (cond ((#/^\"((?:[^\"]|\"\")*)\"/ s)
-           => (lambda (m)
-                (entry (m 'after)
-                       (cons `(delimited ,(regexp-replace-all #/""/ (m 1) "\""))
-                             r))))
-          (else
-           (e "unterminated delimited identifier in SQL: ~s" sql-string))))
+    (scan-quote (string-drop s 1) r #\"
+                (lambda (body rest)
+                  (entry rest
+                         `((delimited ,(regexp-replace-all #/""/ body "\""))
+                           . ,r)))
+                "unterminated delimited identifier in SQL: ~s"))
+
   (define (scan-bitstring s r)
     (cond ((#/^.'([01]+)'/ s)
            => (lambda (m)
