@@ -1,7 +1,7 @@
 ;;;
 ;;; cgi.scm - CGI utility
 ;;;  
-;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: cgi.scm,v 1.28 2005/10/30 14:04:01 shirok Exp $
+;;;  $Id: cgi.scm,v 1.34 2007/04/19 05:54:43 shirok Exp $
 ;;;
 
 ;; Surprisingly, there's no ``formal'' definition of CGI.
@@ -53,6 +53,7 @@
   (use text.tree)
   (use text.html-lite)
   (use file.util)
+  (use util.list)
   (export cgi-metavariables
           cgi-get-metavariable
           cgi-output-character-encoding
@@ -163,7 +164,7 @@
              (or (and-let* ((lenp (or content-length
                                       (get-meta "CONTENT_LENGTH")))
                             (len  (x->integer lenp))
-                            ((positive? len)))
+                            ((<= 0 len)))
                    (string-incomplete->complete (read-block len)))
                  (port->string (current-input-port)))))
           (else
@@ -177,12 +178,12 @@
 ;;                                    part-handlers content-type
 ;;                                    mime-input
 (define (cgi-parse-parameters . args)
-  (let-keywords* args ((query-string #f)
-                       (merge-cookies #f)
-                       (part-handlers '())
-                       (content-type #f)
-                       (content-length #f)
-                       (mime-input #f))
+  (let-keywords args ((query-string #f)
+                      (merge-cookies #f)
+                      (part-handlers '())
+                      (content-type #f)
+                      (content-length #f)
+                      (mime-input #f))
     (let* ((input (cond (query-string)
                         ((or mime-input content-type) 'mime)
                         (else
@@ -336,7 +337,7 @@
                (result (handler name filename part-info inp)))
           (list name result))))))
 
-  (let* ((inp (if (and clength (positive? (x->integer clength)))
+  (let* ((inp (if (and clength (<= 0 (x->integer clength)))
                 (open-input-limited-length-port inp (x->integer clength))
                 inp))
          (result (mime-parse-message inp `(("content-type" ,ctype))
@@ -362,28 +363,25 @@
 ;; API: cgi-header &keyword content-type cookies location &allow-other-keys
 ;;
 (define (cgi-header . args)
-  (let-keywords* args ((content-type #f)
-                       (location     #f)
-                       (status       #f)
-                       (cookies      '()))
+  (let-keywords args ((content-type #f)
+                      (location     #f)
+                      (status       #f)
+                      (cookies      '())
+                      . rest)
     (let ((ct (or content-type
                   (and (not location) "text/html")))
           (r '()))
-      (when status   (push! r #`"Status: ,status\n"))
-      (when ct       (push! r #`"Content-type: ,ct\n"))
-      (when location (push! r #`"Location: ,location\n"))
+      (when status   (push! r #`"Status: ,status\r\n"))
+      (when ct       (push! r #`"Content-type: ,ct\r\n"))
+      (when location (push! r #`"Location: ,location\r\n"))
       (for-each (lambda (cookie)
-                  (push! r #`"Set-cookie: ,cookie\n"))
+                  (push! r #`"Set-cookie: ,cookie\r\n"))
                 cookies)
-      (let loop ((args args))
-        (cond ((null? args))
-              ((null? (cdr args)))
-              ((memv (car args) '(:content-type :location :status :cookies))
-               (loop (cddr args)))
-              (else
-               (push! r #`",(car args): ,(cadr args)\n")
-               (loop (cddr args)))))
-      (push! r "\n")
+      (for-each (lambda (p)
+                  (when (pair? (cdr p))
+                    (push! r #`",(car p): ,(cadr p)\r\n")))
+                (slices rest 2))
+      (push! r "\r\n")
       (reverse r))))
       
 ;;----------------------------------------------------------------
@@ -394,21 +392,13 @@
                        (output-proc cgi-default-output)
                        (merge-cookies #f)
                        (part-handlers '()))
-    ;; we need to keep the output port, for the error handler may be
-    ;; called while the current output port is altered.
-    (let1 default-output (current-output-port)
-      (with-error-handler
-          (lambda (e)
-            (with-output-to-port default-output
-              (cut output-proc (on-error e))))
-        (lambda ()
-          (let1 params
-              (cgi-parse-parameters :merge-cookies merge-cookies
-                                    :part-handlers part-handlers)
-            (output-proc (proc params))))))
+    (guard (e (else (output-proc (on-error e))))
+      (let1 params (cgi-parse-parameters :merge-cookies merge-cookies
+                                         :part-handlers part-handlers)
+        (output-proc (proc params))))
     ;; remove any temporary files
     (for-each sys-unlink (cgi-temporary-files))
-    ))
+    0))
 
 ;; aux fns
 (define (cgi-default-error-proc e)
