@@ -1,7 +1,7 @@
 ;;;
 ;;; collection.scm - collection generics
 ;;;  
-;;;   Copyright (c) 2000-2006 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: collection.scm,v 1.4 2006/10/21 09:39:07 shirok Exp $
+;;;  $Id: collection.scm,v 1.8 2007/07/11 06:57:22 shirok Exp $
 ;;;
 
 ;; Defines generic operations over collection.   A collection is
@@ -43,7 +43,8 @@
           call-with-builder  with-builder
           fold fold2 fold3 map map-to map-accum for-each
           fold$ fold2$ fold3$ map$ for-each$
-          find filter filter-to remove remove-to partition partition-to
+          find find-min find-max find-min&max
+          filter filter-to remove remove-to partition partition-to
           size-of lazy-size-of coerce-to
           group-collection)
   )
@@ -93,6 +94,15 @@
               (begin0 (cons k v)
                       (set!-values (k v) (iter eof-marker))))))))
 
+(define-method call-with-iterator ((coll <tree-map>) proc . args)
+  (let ((eof-marker (cons #f #f))
+        (iter (%tree-map-iter coll)))
+    (receive (k v) (iter eof-marker #f)
+      (proc (cut eq? k eof-marker)
+            (lambda ()
+              (begin0 (cons k v)
+                      (set!-values (k v) (iter eof-marker #f))))))))
+
 ;; n-ary case aux. proc
 (define (call-with-iterators colls proc)
   (let loop ((colls colls)
@@ -167,6 +177,16 @@
             (hash-table-put! h (car item) (cdr item)))
           (lambda () h))))
 
+(define-method call-with-builder ((class <tree-map-meta>) proc . args)
+  (let-keywords* args ((key=? =)
+                       (key<? <))
+    (let1 tree (make-tree-map key=? key<?)
+      (proc (lambda (item)
+              (unless (pair? item)
+                (error "pair required to build a tree-map, but got" item))
+              (tree-map-put! tree (car item) (cdr item)))
+            (lambda () tree)))))
+
 ;; utility.  return minimum size of collections if it's easily known, or #f.
 (define (maybe-minimum-size col more)
   (let1 size (and-let* ((siz (lazy-size-of col))
@@ -379,6 +399,57 @@
 (define-method find (pred (coll <list>))
   ((with-module srfi-1 find) pred coll))
 
+;; find-min, find-max, find-min&max ---------------------
+
+(define-method find-min ((coll <collection>) . keys)
+  (let-keywords* keys ((key identity)
+                       (compare <)
+                       (default #f))
+    (%find-minmax-1 coll key compare default)))
+
+(define-method find-max ((coll <collection>) . keys)
+  (let-keywords* keys ((key identity)
+                       (compare <)
+                       (default #f))
+    (%find-minmax-1 coll key (complement compare) default)))
+
+(define (%find-minmax-1 coll key compare default)
+  (with-iterator (coll end? next)
+    (if (end?)
+      default
+      (let1 elt (next)
+        (let loop ((val (key elt))
+                   (elt elt))
+          (if (end?)
+            elt
+            (let* ((e (next))
+                   (v (key e)))
+              (if (compare v val)
+                (loop v e)
+                (loop val elt)))))))))
+
+(define-method find-min&max ((coll <collection>) . keys)
+  (let-keywords* keys ((key identity)
+                       (compare <)
+                       (default #f)
+                       (default-min default)
+                       (default-max default))
+    (with-iterator (coll end? next)
+      (if (end?)
+        (values default-min default-max)
+        (let1 elt (next)
+          (let loop ((minval (key elt))
+                     (minelt elt)
+                     (maxval (key elt))
+                     (maxelt elt))
+            (if (end?)
+              (values minelt maxelt)
+              (let* ((e (next))
+                     (v (key e)))
+                (cond ((compare v minval) (loop v e maxval maxelt))
+                      ((compare maxval v) (loop minval minelt v e))
+                      (else (loop minval minelt maxval maxelt)))))))))))
+
 ;; filter -----------------------------------------------
 
 ;; generic way
@@ -480,8 +551,8 @@
 ;;  cf. group-sequence in gauche.sequence
 
 (define-method group-collection ((col <collection>) . args)
-  (let-keywords* args ((key-proc  :key identity)
-                       (test-proc :test  eqv?))
+  (let-keywords args ((key-proc  :key identity)
+                      (test-proc :test  eqv?))
     (fold (lambda (b r) (cons (reverse! (cdr b)) r))
           '()
           (fold (lambda (elt buckets)
