@@ -1,7 +1,7 @@
 ;;;
 ;;; srfi-13.scm - string library
 ;;;  
-;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;;   
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: srfi-13.scm,v 1.2 2005/09/07 02:09:51 shirok Exp $
+;;;  $Id: srfi-13.scm,v 1.7 2007/08/01 08:03:31 shirok Exp $
 ;;;
 
 ;; Natively implemented functions:
@@ -139,13 +139,20 @@
 
 (define (string-copy! target tstart s . args)
   (check-arg string? target)
-  (check-arg (lambda (x) (and (integer? x) (positive? x))) tstart)
+  (check-arg (lambda (x) (and (integer? x) (>= x 0))) tstart)
   (let* ((str (apply %maybe-substring s args))
          (slen (string-length str))
          (tlen (string-length target)))
     (when (> (+ tstart slen) tlen)
       (error "copy operation runs off the target string:" target))
-    (string-substitute! target tstart str)))
+    (if (= slen tlen)
+      (%string-replace-body! target str)
+      (%string-replace-body! target
+                             (string-append (substring target 0 tstart)
+                                            str
+                                            (substring target
+                                                       (+ tstart slen)
+                                                       tlen))))))
 
 (define (string-pad s len . args)
   (let-optionals* args ((char #\space) start end)
@@ -580,25 +587,24 @@
       (cond ((eof-object? ch) (get-output-string dst))
             ((char-set-contains? *%cased-char-set* ch)
              (if title?
-                 (write-char (char-upcase ch) dst) ;; This should be char-titlecase
-                 (write-char (char-downcase ch) dst))
+               (write-char (char-upcase ch) dst) ;; This should be char-titlecase
+               (write-char (char-downcase ch) dst))
              (loop #f (read-char src)))
             (else
              (write-char ch dst)
              (loop #t (read-char src))))
       )))
 
-(define (string-upcase! s . args)
-  (let-optionals* args ((start 0) end)
-    (string-substitute! s start (string-upcase s start end))))
+(define (%string-*case! converter)
+  (lambda (s . args)
+    (let-optionals* args ((start 0) (end (string-length s)))
+      (if (and (= start 0) (= end (string-length s)))
+        (%string-replace-body! s (converter s))
+        (string-copy! s start (converter s start end))))))
 
-(define (string-downcase! s . args)
-  (let-optionals* args ((start 0) end)
-    (string-substitute! s start (string-downcase s start end))))
-
-(define (string-titlecase! s . args)
-  (let-optionals* args ((start 0) end)
-    (string-substitute! s start (string-titlecase s start end))))
+(define string-upcase!    (%string-*case! string-upcase))
+(define string-downcase!  (%string-*case! string-downcase))
+(define string-titlecase! (%string-*case! string-titlecase))
 
 ;;;
 ;;; Reverse & append
@@ -609,15 +615,15 @@
         (dst (open-output-string)))
     (let loop ((ch (string-pointer-prev! sp)))
       (if (eof-object? ch)
-          (get-output-string dst)
-          (begin (write-char ch dst)
-                 (loop (string-pointer-prev! sp)))))
+        (get-output-string dst)
+        (begin (write-char ch dst)
+               (loop (string-pointer-prev! sp)))))
     ))
 
 (define (string-reverse! s . args)
   (let-optionals* args ((start 0) end)
     (let ((rev (apply string-reverse s args)))
-      (string-substitute! s start rev))))
+      (string-copy! s start rev))))
 
 (define (string-concatenate lis)
   (cond
@@ -668,9 +674,9 @@
         (dest (open-output-string)))
     (let loop ((ch (string-pointer-next! src)))
       (if (eof-object? ch)
-          (get-output-string dest)
-          (begin (write-char (proc ch) dest)
-                 (loop (string-pointer-next! src)))))
+        (get-output-string dest)
+        (begin (write-char (proc ch) dest)
+               (loop (string-pointer-next! src)))))
     ))
 
 (define (string-map! proc s . args)
@@ -678,7 +684,7 @@
   (check-arg string? s)
   (let-optionals* args ((start 0) end)
      (let ((mapped (apply string-map proc s args)))
-       (string-substitute! s start mapped))))
+       (string-copy! s start mapped))))
 
 (define (string-fold kons knil s . args)
   (check-arg procedure? kons)
@@ -687,8 +693,8 @@
     (let loop ((ch (string-pointer-next! src))
                (r  knil))
       (if (eof-object? ch)
-          r
-          (loop (string-pointer-next! src) (kons ch r))))
+        r
+        (loop (string-pointer-next! src) (kons ch r))))
     ))
 
 (define (string-fold-right kons knil s . args)
@@ -698,8 +704,8 @@
     (let loop ((ch (string-pointer-prev! src))
                (r  knil))
       (if (eof-object? ch)
-          r
-          (loop (string-pointer-prev! src) (kons ch r))))
+        r
+        (loop (string-pointer-prev! src) (kons ch r))))
     ))
 
 (define (string-unfold p f g seed . args)
@@ -711,10 +717,10 @@
       (display base dest)
       (let loop ((seed seed))
         (if (p seed)
-            (begin (display (make-final seed) dest)
-                   (get-output-string dest))
-            (begin (write-char (f seed) dest)
-                   (loop (g seed))))))
+          (begin (display (make-final seed) dest)
+                 (get-output-string dest))
+          (begin (write-char (f seed) dest)
+                 (loop (g seed))))))
     ))
 
 (define (string-unfold-right p f g seed . args)
@@ -725,11 +731,11 @@
     (let ((dest (open-output-string)))
       (let loop ((seed seed))
         (if (p seed)
-            (string-append (make-final seed)
-                           (string-reverse (get-output-string dest))
-                           base)
-            (begin (write-char (f seed) dest)
-                   (loop (g seed))))))
+          (string-append (make-final seed)
+                         (string-reverse (get-output-string dest))
+                         base)
+          (begin (write-char (f seed) dest)
+                 (loop (g seed))))))
     ))
 
 (define (string-for-each proc s . args)
@@ -795,7 +801,7 @@
   (check-arg string? target)
   (check-arg (lambda (x) (and (integer? x) (exact? x))) tstart)
   (let ((result (apply xsubstring s sfrom args)))
-    (string-substitute! target tstart result)))
+    (string-copy! target tstart result)))
 
 ;;;
 ;;; Miscellaneous
@@ -866,17 +872,17 @@
   (let ((slen (string-length s)))
     (let-optionals* args (start end)
       (if (undefined? start)
-          (values '() 0 slen)
-          (if (not (and (integer? start) (exact? start) (<= 0 start slen)))
-              (errorf "~s: argument out of range: ~s" proc start)
-              (if (undefined? end)
-                  (values '() start slen)
-                  (if (not (and (integer? end) (exact? end) (<= start end slen)))
-                      (errorf "~s: argument out of range: ~s" proc end)
-                      (values (cddr args) start end))
-                  )
-              )
+        (values '() 0 slen)
+        (if (not (and (integer? start) (exact? start) (<= 0 start slen)))
+          (errorf "~s: argument out of range: ~s" proc start)
+          (if (undefined? end)
+            (values '() start slen)
+            (if (not (and (integer? end) (exact? end) (<= start end slen)))
+              (errorf "~s: argument out of range: ~s" proc end)
+              (values (cddr args) start end))
+            )
           )
+        )
       )))
 
 (define (string-parse-final-start+end proc s args)
@@ -884,19 +890,19 @@
   (let ((slen (string-length s)))
     (let-optionals* args (start end)
       (if (undefined? start)
-          (values '() 0 slen)
-          (if (not (and (integer? start) (exact? start) (<= 0 start slen)))
-              (errorf "~s: argument out of range: ~s" proc start)
-              (if (undefined? end)
-                  (values '() start slen)
-                  (if (not (and (integer? end) (exact? end) (<= start end slen)))
-                      (errorf "~s: argument out of range: ~s" proc end)
-                      (if (not (null? (cddr args)))
-                          (errorf "~s: too many arguments" proc)
-                          (values (cddr args) start end)))
-                  )
-              )
+        (values '() 0 slen)
+        (if (not (and (integer? start) (exact? start) (<= 0 start slen)))
+          (errorf "~s: argument out of range: ~s" proc start)
+          (if (undefined? end)
+            (values '() start slen)
+            (if (not (and (integer? end) (exact? end) (<= start end slen)))
+              (errorf "~s: argument out of range: ~s" proc end)
+              (if (not (null? (cddr args)))
+                (errorf "~s: too many arguments" proc)
+                (values (cddr args) start end)))
+            )
           )
+        )
       )))
 
 (define-syntax let-string-start+end
@@ -944,23 +950,23 @@
         (let loop ((k (+ (vector-ref rv i) 1)))
           (if (and (> k 0)
                    (not (c= (string-ref pat i) (string-ref pat (- k 1)))))
-              (loop (+ (vector-ref rv (- k 1)) 1))
-              (vector-set! rv (+ i 1) k)))
+            (loop (+ (vector-ref rv (- k 1)) 1))
+            (vector-set! rv (+ i 1) k)))
         )
       )))
 
 (define (kmp-step pat rv c i c= p-start)
   (let loop ((i i))
     (if (c= c (string-ref pat (+ i p-start)))
-        (+ i 1)
-        (let ((i (vector-ref rv i)))
-          (if (= i -1) 0 (loop i))))))
+      (+ i 1)
+      (let ((i (vector-ref rv i)))
+        (if (= i -1) 0 (loop i))))))
 
 ;; This is inefficient if input string s contains multibyte characters.
 (define (string-kmp-partial-search pat rv s i . args)
   (let-optionals* args ((c= char=?) (p-start 0) start end)
     (let ((patlen (vector-length rv)))
-      (let lp ((si s-start)
+      (let lp ((si start)
                (vi i))
         (cond ((= vi patlen) (- si)) 
               ((= si end) vi)
