@@ -1,7 +1,7 @@
 /*
  * core.c - core kernel interface
  *
- *   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,16 +30,14 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: core.c,v 1.70 2006/11/03 11:11:27 shirok Exp $
+ *  $Id: core.c,v 1.82 2007/09/13 12:30:27 shirok Exp $
  */
 
-#include <stdlib.h>
-#include <unistd.h>
 #define LIBGAUCHE_BODY
-#define GAUCHE_API_0_8_8
 #include "gauche.h"
 #include "gauche/arch.h"
 #include "gauche/paths.h"
+#include "gauche/builtin-syms.h"
 
 /*
  * out-of-memory handler.  this will be called by GC.
@@ -51,6 +49,14 @@ static GC_PTR oom_handler(size_t bytes)
     return NULL;                /* dummy */
 }
 
+/*
+ * Features list used by cond-expand macro
+ */
+static struct {
+    ScmObj alist;
+    ScmInternalMutex mutex;
+} cond_features = { SCM_NIL };
+
 /*=============================================================
  * Program initialization
  */
@@ -77,6 +83,7 @@ extern void Scm__InitVM(void);
 extern void Scm__InitRepl(void);
 extern void Scm__InitParameter(void);
 extern void Scm__InitAutoloads(void);
+extern void Scm__InitCollection(void);
 
 extern void Scm_Init_stdlib(ScmModule *);
 extern void Scm_Init_extlib(ScmModule *);
@@ -89,7 +96,7 @@ extern void Scm_Init_compile(void);
 extern void Scm_Init_objlib(void);
 
 static void finalizable(void);
-
+static void init_cond_features(void);
 
 #ifdef GAUCHE_USE_PTHREADS
 /* a trick to make sure the gc thread object is linked */
@@ -118,6 +125,8 @@ void Scm_Init(const char *signature)
     GC_finalize_on_demand = TRUE;
     GC_finalizer_notifier = finalizable;
 
+    (void)SCM_INTERNAL_MUTEX_INIT(cond_features.mutex);
+
     /* Initialize components.  The order is important, for some components
        rely on the other components to be initialized. */
     Scm__InitSymbol();
@@ -126,6 +135,7 @@ void Scm_Init(const char *signature)
     Scm__InitNumber();
     Scm__InitChar();
     Scm__InitClass();
+    Scm__InitCollection();
     Scm__InitExceptions();
     Scm__InitProc();
     Scm__InitPort();
@@ -158,6 +168,9 @@ void Scm_Init(const char *signature)
 
     Scm_SelectModule(Scm_UserModule());
 
+    /* Final setup of cond-features alist. */
+    init_cond_features();
+
 #ifdef GAUCHE_USE_PTHREADS
     /* a trick to make sure the gc thread object is linked */
     ptr_pthread_create = (int (*)(void))GC_pthread_create;
@@ -168,6 +181,11 @@ void Scm_Init(const char *signature)
  * GC utilities
  */
 
+void Scm_GC()
+{
+    GC_gcollect();
+}
+
 /*
  * External API to register root set in dynamically loaded library.
  * Boehm GC doesn't do this automatically on some platforms.
@@ -334,16 +352,116 @@ void Scm_Panic(const char *msg, ...)
 /* Use this for absolute emergency.  Newline is not attached to msg. */
 void Scm_Abort(const char *msg)
 {
-    int size = strlen(msg);
+    int size = (int)strlen(msg);
     write(2, msg, size); /* this may return an error, but we don't care */
     _exit(1);
 }
 
 /*=============================================================
  * Inspect the configuration
- *
  */
 
+/* Returns the cond-features alist. */
+ScmObj
+Scm_GetFeatures()
+{
+    return cond_features.alist;
+}
+
+/*
+ * Append FEATURE to the cond-features alist.  Once added, the symbol FEATURE
+ * will be recognized by cond-expand.
+ *
+ *  (cond-expand (FEATURE body ...)) 
+ *   
+ * If loading a module is required in order to make FEATURE available,
+ * such module name can be specified in MODULE argument.  It can be NULL
+ * if the feature is built-in.
+ *
+ * This API is meant to expose configuration information to a Scheme
+ * level.
+ */
+void
+Scm_AddFeature(const char *feature, const char *module)
+{
+    ScmObj cell;
+
+    if (module) {
+        cell = SCM_LIST2(SCM_INTERN(feature), SCM_INTERN(module));
+    } else {
+        cell = SCM_LIST1(SCM_INTERN(feature));
+    }
+    
+    (void)SCM_INTERNAL_MUTEX_LOCK(cond_features.mutex);
+    cond_features.alist = Scm_Cons(cell, cond_features.alist);
+    (void)SCM_INTERNAL_MUTEX_UNLOCK(cond_features.mutex);
+}
+
+static void
+init_cond_features()
+{
+    int i;
+    /* The initial cond-features list. */
+    static struct {
+        const char *feature;
+        const char *module;
+    } init_features[] = {
+        { "gauche", NULL },
+#ifdef GAUCHE_WINDOWS
+        { "gauche.os.windows", NULL },
+        { "gauche-windows", NULL }, /* for backward compatibility */
+#endif
+#ifdef GAUCHE_USE_PTHREADS
+        { "gauche.sys.pthreads", "gauche.threads" },
+#endif
+        { "srfi-0", NULL },         /* autoloaded */
+        { "srfi-1", "srfi-1" },
+        { "srfi-2", NULL },         /* builtin */
+        { "srfi-4", "gauche.uvector" },
+        { "srfi-5", "srfi-5" },
+        { "srfi-6", NULL },         /* builtin */
+        { "srfi-7", NULL },         /* autoloaded */
+        { "srfi-8", NULL },         /* builtin */
+        { "srfi-9", "srfi-9" },
+        { "srfi-10", NULL },
+        { "srfi-11", "srfi-11" },
+        { "srfi-13", "srfi-13" },
+        { "srfi-14", "srfi-14" },
+        { "srfi-16", NULL },
+        { "srfi-17", NULL },
+        { "srfi-18", "gauche.threads" },
+        { "srfi-19", "srfi-19" },
+        { "srfi-22", NULL },
+        { "srfi-23", NULL },
+        { "srfi-25", "gauche.array" },
+        { "srfi-26", NULL },
+        { "srfi-27", "srfi-27" },
+        { "srfi-28", NULL },
+        { "srfi-29", "srfi-29" },
+        { "srfi-30", NULL },
+        { "srfi-31", NULL },
+        { "srfi-34", NULL },
+        { "srfi-35", NULL },
+        { "srfi-36", NULL },
+        { "srfi-37", "srfi-37" },
+        { "srfi-38", NULL },
+        { "srfi-39", "gauche.parameter" },
+        { "srfi-40", "util.stream" },
+        { "srfi-42", "srfi-42" },
+        { "srfi-43", "srfi-43" },
+        { "srfi-45", NULL },
+        { "srfi-55", NULL },
+        { "srfi-61", NULL },
+        { "srfi-62", NULL },
+        { "srfi-87", NULL },
+        { NULL, NULL }
+    };
+
+    for (i=0; init_features[i].feature; i++) {
+        Scm_AddFeature(init_features[i].feature, init_features[i].module);
+    }
+}
+
 const char *Scm_HostArchitecture(void)
 {
     return GAUCHE_ARCH;
@@ -360,7 +478,7 @@ ScmObj Scm_LibraryDirectory(void)
         char buf[PATH_MAX];
         Scm_GetLibraryDirectory(buf, PATH_MAX, Scm_Error);
         dir = Scm_MakeString(buf, -1, -1,
-                             SCM_MAKSTR_COPYING|SCM_MAKSTR_IMMUTABLE);
+                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
     }
     return dir;
 }
@@ -372,7 +490,7 @@ ScmObj Scm_ArchitectureDirectory(void)
         char buf[PATH_MAX];
         Scm_GetArchitectureDirectory(buf, PATH_MAX, Scm_Error);
         dir = Scm_MakeString(buf, -1, -1,
-                             SCM_MAKSTR_COPYING|SCM_MAKSTR_IMMUTABLE);
+                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
     }
     return dir;
 }
@@ -384,7 +502,7 @@ ScmObj Scm_SiteLibraryDirectory(void)
         char buf[PATH_MAX];
         Scm_GetSiteLibraryDirectory(buf, PATH_MAX, Scm_Error);
         dir = Scm_MakeString(buf, -1, -1,
-                             SCM_MAKSTR_COPYING|SCM_MAKSTR_IMMUTABLE);
+                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
     }
     return dir;
 }
@@ -396,7 +514,7 @@ ScmObj Scm_SiteArchitectureDirectory(void)
         char buf[PATH_MAX];
         Scm_GetSiteArchitectureDirectory(buf, PATH_MAX, Scm_Error);
         dir = Scm_MakeString(buf, -1, -1,
-                             SCM_MAKSTR_COPYING|SCM_MAKSTR_IMMUTABLE);
+                             SCM_STRING_COPYING|SCM_STRING_IMMUTABLE);
     }
     return dir;
 }
