@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: gauche.h,v 1.502 2007/09/13 12:30:27 shirok Exp $
+ *  $Id: gauche.h,v 1.505 2008/02/08 11:53:33 shirok Exp $
  */
 
 #ifndef GAUCHE_H
@@ -685,8 +685,10 @@ SCM_CLASS_DECL(Scm_ForeignPointerClass);
 #define SCM_CLASS_OBJECT           (&Scm_ObjectClass)
 #define SCM_CLASS_FOREIGN_POINTER  (&Scm_ForeignPointerClass)
 
-SCM_EXTERN ScmClass *Scm_DefaultCPL[];
-SCM_EXTERN ScmClass *Scm_ObjectCPL[];
+/* NB: we can't use SCM_EXTERN because Windows DLL can't use the address of
+   dllimport-ed variables as constants. */
+extern ScmClass *Scm_DefaultCPL[];
+extern ScmClass *Scm_ObjectCPL[];
 
 #define SCM_CLASS_DEFAULT_CPL     (Scm_DefaultCPL)
 #define SCM_CLASS_OBJECT_CPL      (Scm_ObjectCPL)
@@ -1349,8 +1351,10 @@ SCM_EXTERN int    Scm_HasSetter(ScmObj proc);
 /* Generic - Generic function */
 struct ScmGenericRec {
     ScmProcedure common;
-    ScmObj methods;
-    ScmObj (*fallback)(ScmObj *args, int nargs, ScmGeneric *gf);
+    ScmObj methods;             /* list of methods */
+    int   maxReqargs;           /* maximum # of args required to select
+                                   applicable methods */
+    ScmObj (*fallback)(ScmObj *argv, int argc, ScmGeneric *gf);
     void *data;
     ScmInternalMutex lock;
 };
@@ -1366,7 +1370,7 @@ SCM_CLASS_DECL(Scm_GenericClass);
         SCM__PROCEDURE_INITIALIZER(SCM_CLASS_STATIC_TAG(Scm_GenericClass),\
                                    0, 0, SCM_PROC_GENERIC, SCM_FALSE,   \
                                    NULL),                               \
-        SCM_NIL, cfunc, data                                            \
+        SCM_NIL, 0, cfunc, data                                         \
     }
 
 SCM_EXTERN void Scm_InitBuiltinGeneric(ScmGeneric *gf, const char *name,
@@ -1374,9 +1378,9 @@ SCM_EXTERN void Scm_InitBuiltinGeneric(ScmGeneric *gf, const char *name,
 SCM_EXTERN ScmObj Scm_MakeBaseGeneric(ScmObj name,
 				      ScmObj (*fallback)(ScmObj *, int, ScmGeneric*),
 				      void *data);
-SCM_EXTERN ScmObj Scm_NoNextMethod(ScmObj *args, int nargs, ScmGeneric *gf);
-SCM_EXTERN ScmObj Scm_NoOperation(ScmObj *args, int nargs, ScmGeneric *gf);
-SCM_EXTERN ScmObj Scm_InvalidApply(ScmObj *args, int nargs, ScmGeneric *gf);
+SCM_EXTERN ScmObj Scm_NoNextMethod(ScmObj *argv, int argc, ScmGeneric *gf);
+SCM_EXTERN ScmObj Scm_NoOperation(ScmObj *argv, int argc, ScmGeneric *gf);
+SCM_EXTERN ScmObj Scm_InvalidApply(ScmObj *argv, int argc, ScmGeneric *gf);
 
 /* Method - method
    A method can be defined either by C or by Scheme.  C-defined method
@@ -1386,7 +1390,7 @@ struct ScmMethodRec {
     ScmProcedure common;
     ScmGeneric *generic;
     ScmClass **specializers;    /* array of specializers, size==required */
-    ScmObj (*func)(ScmNextMethod *nm, ScmObj *args, int nargs, void * data);
+    ScmObj (*func)(ScmNextMethod *nm, ScmObj *argv, int argc, void * data);
     void *data;                 /* closure, or code */
     ScmEnvFrame *env;           /* environment (for Scheme created method) */
 };
@@ -1412,9 +1416,10 @@ SCM_EXTERN void Scm_InitBuiltinMethod(ScmMethod *m);
 struct ScmNextMethodRec {
     ScmProcedure common;
     ScmGeneric *generic;
-    ScmObj methods;             /* list of applicable methods */
-    ScmObj *args;               /* original arguments */
-    int nargs;                  /* # of original arguments */
+    ScmObj methods;          /* list of applicable methods */
+    ScmObj *argv;            /* original arguments */
+    int argc;                /* # of original arguments */
+    int applyargs;           /* if TRUE, argv[argc-1] has a list of rest args */
 };
 
 SCM_CLASS_DECL(Scm_NextMethodClass);
