@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: load.c,v 1.116 2007/09/15 12:30:50 shirok Exp $
+ *  $Id: load.c,v 1.118 2008/02/12 14:24:10 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -1046,8 +1046,21 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
     /* shortcut in case if somebody else already did the job. */
     if (adata->loaded) return adata->value;
 
-    /* check to see if this autoload is recursive. */
-    if (!SCM_FALSEP(Scm_Assoc(SCM_OBJ(adata->path), ldinfo.providing, SCM_CMP_EQUAL))) {
+    /* check to see if this autoload is recursive.  if so, we just return
+       SCM_UNBOUND and let the caller handle the issue (NB: it isn't
+       necessarily an error.  For example, define-method searches if
+       a generic function of the same name is already defined; if the
+       name is set autoload and define-method is in the file that's being
+       autoloaded, define-method finds the name is an autoload that points
+       the currently autoloaded file.)
+       we have to be careful to exclude the case that when one thread is
+       resolving autoload another thread enters here and sees this autoload
+       is already being resolved.
+     */
+    if ((adata->locker == NULL || adata->locker == vm)
+        && !SCM_FALSEP(Scm_Assoc(SCM_OBJ(adata->path),
+                                 ldinfo.providing,
+                                 SCM_CMP_EQUAL))) {
         return SCM_UNBOUND;
     }
 
@@ -1080,7 +1093,7 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
            if we reach here.  Right now I have no idea how this happens, but
            just in case we raise an error. */
         adata->locker = NULL;
-        SCM_INTERNAL_COND_SIGNAL(adata->cv);
+        SCM_INTERNAL_COND_BROADCAST(adata->cv);
         Scm_Error("Attempted to trigger the same autoload %S#%S recursively.  Maybe circular autoload dependency?\n",
                   adata->module, adata->name);
     }
@@ -1128,13 +1141,13 @@ ScmObj Scm_ResolveAutoload(ScmAutoload *adata, int flags)
     } SCM_WHEN_ERROR {
         adata->locker = NULL;
         vm->module = prev_module;
-        SCM_INTERNAL_COND_SIGNAL(adata->cv);
+        SCM_INTERNAL_COND_BROADCAST(adata->cv);
         SCM_NEXT_HANDLER;
     } SCM_END_PROTECT;
 
     adata->loaded = TRUE;
     adata->locker = NULL;
-    SCM_INTERNAL_COND_SIGNAL(adata->cv);
+    SCM_INTERNAL_COND_BROADCAST(adata->cv);
     return adata->value;
 }
 
