@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: read.c,v 1.95 2007/09/07 09:51:13 shirok Exp $
+ *  $Id: read.c,v 1.98 2008/02/02 05:14:42 shirok Exp $
  */
 
 #include <stdio.h>
@@ -721,48 +721,88 @@ static ScmObj read_string(ScmPort *port, int incompletep,
     for (;;) {
         FETCH(c);
         switch (c) {
-          case EOF: goto eof_exit;
-          case '"': {
-              int flags = ((incompletep? SCM_STRING_INCOMPLETE : 0)
-                           | SCM_STRING_IMMUTABLE);
-              return Scm_DStringGet(&ds, flags);
-          }
-          case '\\': {
-            int c1 = Scm_GetcUnsafe(port);
-            switch (c1) {
-              case EOF: goto eof_exit;
-              case 'n': ACCUMULATE('\n'); break;
-              case 'r': ACCUMULATE('\r'); break;
-              case 'f': ACCUMULATE('\f'); break;
-              case 't': ACCUMULATE('\t'); break;
-              case '\\': ACCUMULATE('\\'); break;
-              case '0': ACCUMULATE(0); break;
-              case 'x': {
-                  int cc = read_string_xdigits(port, 2, 'x', incompletep);
-                  ACCUMULATE(cc);
-                  break;
-              }
-              case 'u': {
-                  int cc = read_string_xdigits(port, 4, 'u', incompletep);
-                  ACCUMULATE(Scm_UcsToChar(cc));
-                  break;
-              }
-              case 'U': {
-                  int cc = read_string_xdigits(port, 8, 'U', incompletep);
-                  ACCUMULATE(Scm_UcsToChar(cc));
-                  break;
-              }
-              default:
-                ACCUMULATE(c1); break;
+        case EOF: goto eof_exit;
+        case '"': {
+            int flags = ((incompletep? SCM_STRING_INCOMPLETE : 0)
+                         | SCM_STRING_IMMUTABLE);
+            return Scm_DStringGet(&ds, flags);
+        }
+        backslash:
+        case '\\': {
+            FETCH(c);
+            switch (c) {
+            case EOF: goto eof_exit;
+            case 'n': ACCUMULATE('\n'); break;
+            case 'r': ACCUMULATE('\r'); break;
+            case 'f': ACCUMULATE('\f'); break;
+            case 't': ACCUMULATE('\t'); break;
+            case '\\': ACCUMULATE('\\'); break;
+            case '0': ACCUMULATE(0); break;
+            case 'x': {
+                int cc = read_string_xdigits(port, 2, 'x', incompletep);
+                ACCUMULATE(cc);
+                break;
+            }
+            case 'u': {
+                int cc = read_string_xdigits(port, 4, 'u', incompletep);
+                ACCUMULATE(Scm_UcsToChar(cc));
+                break;
+            }
+            case 'U': {
+                int cc = read_string_xdigits(port, 8, 'U', incompletep);
+                ACCUMULATE(Scm_UcsToChar(cc));
+                break;
+            }
+                /* R6RS-style line continuation handling*/
+            case ' ':
+            case '\t': {
+                for (;;) {
+                    FETCH(c);
+                    if (c == EOF) goto eof_exit;
+                    if (c == '\r') goto cont_cr;
+                    if (c == '\n') goto cont_lf;
+                    if (c != ' ' && c != '\t') goto cont_err;
+                }
+            }
+                /*FALLTHROUGH*/
+            cont_cr:
+            case '\r': {
+                FETCH(c);
+                if (c == EOF)  goto eof_exit;
+                if (c == '\\') goto backslash;
+                if (c != '\n' && c != ' ' && c != '\t') {
+                    ACCUMULATE(c);
+                    break;
+                }
+                /*FALLTHROUGH*/
+            }
+            cont_lf:
+            case '\n': {
+                for (;;) {
+                    FETCH(c);
+                    if (c == EOF) goto eof_exit;
+                    if (c == '\\') goto backslash;
+                    if (c != ' ' && c != '\t') {
+                        ACCUMULATE(c);
+                        break;
+                    }
+                }
+                break;
+            }
+            default:
+                ACCUMULATE(c); break;
             }
             break;
-          }
-          default: ACCUMULATE(c); break;
+        }       
+        default: ACCUMULATE(c); break;
         }
     }
  eof_exit:
     Scm_ReadError(port, "EOF encountered in a string literal: %S",
                   Scm_DStringGet(&ds, 0));
+ cont_err:
+    Scm_ReadError(port, "Invalid line continuation sequence in a string literal: %S...",
+                  Scm_DStringGet(&ds, 0));
     /* NOTREACHED */
     return SCM_FALSE; 
 }
@@ -828,7 +868,7 @@ static ScmObj read_char(ScmPort *port, ScmReadContext *ctx)
         /* handle #\uxxxx or #\uxxxxxxxx*/
         if ((cname[0] == 'u') && isxdigit(cname[1])) {
             int code;
-            if (namesize == 5 || namesize == 9) {
+            if (namesize >= 5 && namesize <= 9) {
                 code = Scm_ReadXdigitsFromString(cname+1, namesize-1, NULL);
                 if (code >= 0) return SCM_MAKE_CHAR(Scm_UcsToChar(code));
             }
@@ -946,7 +986,7 @@ static ScmObj read_escaped_symbol(ScmPort *port, ScmChar delim)
 /* gauche extension :  #/regexp/ */
 static ScmObj read_regexp(ScmPort *port)
 {
-    ScmChar c = 0;
+    ScmChar c = 0, c1 = 0;
     ScmDString ds;
     Scm_DStringInit(&ds);
     for (;;) {
@@ -955,12 +995,14 @@ static ScmObj read_regexp(ScmPort *port)
             Scm_ReadError(port, "unterminated literal regexp");
         }
         if (c == '\\') {
-            SCM_DSTRING_PUTC(&ds, c);
-            c = Scm_GetcUnsafe(port);
-            if (c == SCM_CHAR_INVALID) {
+            /* NB: We "eat" a backslash before '/', since it is only dealt
+               with the reader and nothing to do with regexp parser itself. */
+            c1 = Scm_GetcUnsafe(port);
+            if (c1 == SCM_CHAR_INVALID) {
                 Scm_ReadError(port, "unterminated literal regexp");
             }
-            SCM_DSTRING_PUTC(&ds, c);
+            if (c1 != '/') SCM_DSTRING_PUTC(&ds, c);
+            SCM_DSTRING_PUTC(&ds, c1);
         } else if (c == '/') {
             /* Read one more char to see if we have a flag */
             int flags = 0;
