@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;  
-;;;  $Id: scmlib.scm,v 1.14 2007/07/01 08:03:09 shirok Exp $
+;;;  $Id: scmlib.scm,v 1.17 2008/02/06 06:44:07 shirok Exp $
 ;;;
 
 ;; This file contains builtin library functions that are easier to be
@@ -217,6 +217,8 @@
 (define-reader-ctor 'string-interpolate
   (lambda (s) (string-interpolate s))) ;;lambda is required to delay loading
 
+
+
 ;;;=======================================================
 ;;; call/cc alias
 ;;;
@@ -313,7 +315,7 @@
 (define write/ss write-with-shared-structure)
 
 ;;;=======================================================
-;;; i/o utility
+;;; i/o utilities
 ;;;
 
 (define (print . args) (for-each display args) (newline))
@@ -341,6 +343,114 @@
            )
     (values format format/ss)))
 
+;;;=======================================================
+;;; regexp utilities
+;;;
+
+(define-values (regexp-replace regexp-replace-all)
+  (let ()
+    ;; aux routine for regexp-replace[-all]
+    ;; "abc\\1de\\3" => '("abc" 1 "de" 3)
+    (define (regexp-parse-subpattern sub)
+      (cond
+       ((string? sub)
+        (let loop ((sub sub) (r '()))
+          (cond ((rxmatch #/\\(?:(\d+)|k<([^>]+)>|(.))/ sub)
+                 => (lambda (m)
+                      (define (loop2 elem)
+                        (loop (rxmatch-after m)
+                              (list* elem (rxmatch-before m) r)))
+                      (cond ((rxmatch-substring m 1)
+                             => (lambda (d) (loop2 (string->number d))))
+                            ((rxmatch-substring m 2)
+                             => (lambda (s) (loop2 (string->symbol s))))
+                            (else
+                             (loop2 (rxmatch-substring m 3))))))
+                (else (reverse (cons sub r))))))
+       ((procedure? sub) sub)
+       (else (error "string or procedure required, but got" sub))))
+
+    ;; internal loop
+    (define (regexp-replace-rec match subpat out rec)
+      (display (rxmatch-before match) out)
+      (if (procedure? subpat)
+        (display (subpat match) out)
+        (for-each (lambda (pat)
+                    (display (if (or (number? pat) (symbol? pat))
+                               (rxmatch-substring match pat)
+                               pat)
+                             out))
+                  subpat))
+      (rec (rxmatch-after match)))
+
+    (define (regexp-replace rx string sub)
+      (let ((subpat (regexp-parse-subpattern sub))
+            (match  (rxmatch rx string)))
+        (if match
+          (call-with-output-string
+            (lambda (out)
+              (regexp-replace-rec match subpat out
+                                  (lambda (str) (display str out)))))
+          string)))
+
+    ;; The inner call is awkward to avoid creation of output string
+    ;; when no match at all.
+    (define (regexp-replace-all rx string sub)
+      (let ((subpat (regexp-parse-subpattern sub))
+            (match  (rxmatch rx string)))
+        (if match
+          (call-with-output-string
+            (lambda (out)
+              (define (loop str)
+                (unless (equal? str "")
+                  (cond ((rxmatch rx str)
+                         => (lambda (match)
+                              (when (= (rxmatch-start match) (rxmatch-end match))
+                                (error "regexp-replace-all: matching zero-length string causes infinite loop:" rx))
+                              (regexp-replace-rec match subpat out loop)))
+                        (else (display str out)))))
+              (regexp-replace-rec match subpat out loop)))
+          string)))
+    (values regexp-replace regexp-replace-all)))
+
+;; Multiple replacement
+(define-values (regexp-replace* regexp-replace-all*)
+  (let ()
+    (define (regexp-replace-driver name func-1)
+      (lambda (string rx sub . more)
+        (cond ((null? more)
+               (func-1 rx string sub))
+              (else
+               (unless (zero? (modulo (length more) 2))
+                 (errorf "~a: regexp and subsitution don't pair up" name))
+               (let loop ((s (func-1 rx string sub))
+                          (args more))
+                 (if (null? args)
+                   s
+                   (loop (func-1 (car args) s (cadr args))
+                         (cddr args))))))))
+    (values
+     (regexp-replace-driver 'regexp-replace* regexp-replace)
+     (regexp-replace-driver 'regexp-replace-all* regexp-replace-all))))
+
+;; Contributed from Alex Shinn; modified a bit by shiro
+(define (regexp-quote str)
+  (with-string-io str
+    (lambda ()
+      (let loop ((c (read-char)))
+        (unless (eof-object? c)
+          (when (char-set-contains? #[\\|\[\](){}.*+?^$] c) (write-char #\\))
+          (write-char c)
+          (loop (read-char)))))))
+
+(define (rxmatch->string rx str . sel)
+  (cond [(null? sel) (rxmatch-substring (rxmatch rx str))]
+        [(eq? (car sel) 'after)
+         (apply rxmatch-after (rxmatch rx str) (cdr sel))]
+        [(eq? (car sel) 'before)
+         (apply rxmatch-before (rxmatch rx str) (cdr sel))]
+        [else (rxmatch-substring (rxmatch rx str) (car sel))]))
+
 ;;;=======================================================
 ;;; with-something
 ;;;
