@@ -30,7 +30,7 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: system.c,v 1.101 2007/10/29 22:30:38 shirok Exp $
+ *  $Id: system.c,v 1.106 2008/02/05 10:53:50 shirok Exp $
  */
 
 #define LIBGAUCHE_BODY
@@ -44,6 +44,7 @@
 #include <string.h>
 #include <sys/types.h>
 #include <sys/stat.h>
+#include <ctype.h>
 #include <fcntl.h>
 #include <math.h>
 #if !defined(_MSC_VER)
@@ -57,6 +58,8 @@
 #include <lm.h>
 #include <tlhelp32.h>
 static HANDLE *win_prepare_handles(int *fds);
+static int win_wait_for_handles(HANDLE *handles, int nhandles, int options,
+				int *status /*out*/);
 #endif  /* GAUCHE_WINDOWS */
 
 #ifdef HAVE_GLOB_H
@@ -73,23 +76,21 @@ static HANDLE *win_prepare_handles(int *fds);
 
 /*===============================================================
  * Conversion between off_t and Scheme integer.
- * off_t might be either 32bit or 64bit.  However, as far as I know,
- * on ILP32 machines off_t is kept 32bits for compabitility and
- * a separate off64_t is defined for 64bit offset access.
- * To aim completeness I have to support the case that
- * sizeof(off_t) > sizeof(long).  For the time being, I just signal
- * an error outside the long value.
+ * off_t can be either 32bit or 64bit.  
  */
 off_t Scm_IntegerToOffset(ScmObj i)
 {
     if (SCM_INTP(i)) {
         return (off_t)SCM_INT_VALUE(i);
     } else if (SCM_BIGNUMP(i)) {
-        if (SCM_BIGNUM_SIZE(i) > 1
-            || SCM_BIGNUM(i)->values[0] > LONG_MAX) {
-            Scm_Error("offset value too large: %S", i);
-        }
-        return (off_t)Scm_GetInteger(i);
+#if SIZEOF_OFF_T == SIZEOF_LONG
+        return (off_t)Scm_GetIntegerClamp(i, SCM_CLAMP_ERROR, NULL);
+#elif SIZEOF_OFF_T == 8 && !SCM_EMULATE_INT64
+        return (off_t)Scm_GetInteger64Clamp(i, SCM_CLAMP_ERROR, NULL);
+#else
+        /* I don't think there's such an architecture. */
+# error "off_t size on this platform is not suported."
+#endif
     }
     Scm_Error("bad value as offset: %S", i);
     return (off_t)-1;       /* dummy */
@@ -99,13 +100,10 @@ ScmObj Scm_OffsetToInteger(off_t off)
 {
 #if SIZEOF_OFF_T == SIZEOF_LONG
     return Scm_MakeInteger(off);
+#elif SIZEOF_OFF_T == 8 && !SCM_EMULATE_INT64
+    return Scm_MakeInteger64((ScmInt64)off);
 #else
-    if (off <= LONG_MAX && off >= LONG_MIN) {
-        return Scm_MakeInteger(off);
-    } else {
-        Scm_Error("offset value too large to support");
-        return Scm_MakeInteger(-1); /* dummy */
-    }
+# error "off_t size on this platform is not suported."
 #endif
 }
 
@@ -613,24 +611,24 @@ int Scm_Mkstemp(char *templat)
     }
 #define MKSTEMP_MAX_TRIALS 65535   /* avoid infinite loop */
     {
-	u_long seed = (u_long)time(NULL);
-	int numtry, flags;
-	char suffix[7];
+        u_long seed = (u_long)time(NULL);
+        int numtry, flags;
+        char suffix[7];
 #if defined(GAUCHE_WINDOWS)
-	flags = O_CREAT|O_EXCL|O_WRONLY|O_BINARY;
+        flags = O_CREAT|O_EXCL|O_WRONLY|O_BINARY;
 #else  /* !GAUCHE_WINDOWS */
-	flags = O_CREAT|O_EXCL|O_WRONLY;
+        flags = O_CREAT|O_EXCL|O_WRONLY;
 #endif /* !GAUCHE_WINDOWS */
-	for (numtry=0; numtry<MKSTEMP_MAX_TRIALS; numtry++) {
-	    snprintf(suffix, 7, "%06x", seed&0xffffff);
-	    memcpy(templat+siz-6, suffix, 7);
-	    SCM_SYSCALL(fd, open(templat, flags, 0600));
-	    if (fd >= 0) break;
-	    seed *= 2654435761UL;
-	}
-	if (numtry == MKSTEMP_MAX_TRIALS) {
-	    Scm_Error("mkstemp failed");
-	}
+        for (numtry=0; numtry<MKSTEMP_MAX_TRIALS; numtry++) {
+            snprintf(suffix, 7, "%06lx", seed&0xffffff);
+            memcpy(templat+siz-6, suffix, 7);
+            SCM_SYSCALL(fd, open(templat, flags, 0600));
+            if (fd >= 0) break;
+            seed *= 2654435761UL;
+        }
+        if (numtry == MKSTEMP_MAX_TRIALS) {
+            Scm_Error("mkstemp failed");
+        }
     }
     return fd;
 #endif /*!defined(HAVE_MKSTEMP)*/
@@ -700,6 +698,12 @@ static ScmObj stat_perm_get(ScmSysStat *stat)
     return Scm_MakeIntegerFromUI(stat->statrec.st_mode & 0777);
 }
 
+static ScmObj stat_size_get(ScmSysStat *stat)
+{
+    return Scm_OffsetToInteger(stat->statrec.st_size);
+}
+
+
 #define STAT_GETTER_UI(name) \
   static ScmObj SCM_CPP_CAT3(stat_, name, _get)(ScmSysStat *s) \
   { return Scm_MakeIntegerFromUI((u_long)s->statrec.SCM_CPP_CAT(st_, name)); }
@@ -714,7 +718,6 @@ STAT_GETTER_UI(rdev)
 STAT_GETTER_UI(nlink)
 STAT_GETTER_UI(uid)
 STAT_GETTER_UI(gid)
-STAT_GETTER_UI(size) /*TODO: check portability of off_t (maybe 64bits)*/
 STAT_GETTER_TIME(atime)
 STAT_GETTER_TIME(mtime)
 STAT_GETTER_TIME(ctime)
@@ -752,14 +755,15 @@ static ScmObj time_allocate(ScmClass *klass, ScmObj initargs)
     ScmTime *t = SCM_ALLOCATE(ScmTime, klass);
     SCM_SET_CLASS(t, SCM_CLASS_TIME);
     t->type = SCM_SYM_TIME_UTC;
-    t->sec = t->nsec = 0;
+    SCM_SET_INT64_ZERO(t->sec);
+    t->nsec = 0;
     return SCM_OBJ(t);
 }
 
 static void time_print(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
     ScmTime *t = SCM_TIME(obj);
-    Scm_Printf(port, "#<%S %lu.%09lu>", t->type, t->sec, t->nsec);
+    Scm_Printf(port, "#<%S %S.%09lu>", t->type, Scm_MakeInteger64(t->sec), t->nsec);
 }
 
 static int time_compare(ScmObj x, ScmObj y, int equalp)
@@ -769,7 +773,7 @@ static int time_compare(ScmObj x, ScmObj y, int equalp)
     
     if (equalp) {
         if (SCM_EQ(tx->type, ty->type)
-            && tx->sec == ty->sec
+            && SCM_INT64_EQV(tx->sec, ty->sec)
             && tx->nsec == ty->nsec) {
             return 0;
         } else {
@@ -779,8 +783,8 @@ static int time_compare(ScmObj x, ScmObj y, int equalp)
         if (!SCM_EQ(tx->type, ty->type)) {
             Scm_Error("cannot compare different types of time objects: %S vs %S", x, y);
         }
-        if (tx->sec < ty->sec) return -1;
-        if (tx->sec == ty->sec) {
+        if (SCM_INT64_CMP(<, tx->sec, ty->sec)) return -1;
+        if (SCM_INT64_EQV(tx->sec, ty->sec)) {
             if (tx->nsec < ty->nsec) return -1;
             if (tx->nsec == ty->nsec) return 0;
             else return 1;
@@ -793,10 +797,25 @@ SCM_DEFINE_BUILTIN_CLASS(Scm_TimeClass,
                          time_print, time_compare, NULL,
                          time_allocate, SCM_CLASS_DEFAULT_CPL);
 
-ScmObj Scm_MakeTime(ScmObj type, long sec, long nsec)
+static ScmTime *make_time_int(ScmObj type)
 {
     ScmTime *t = SCM_TIME(time_allocate(SCM_CLASS_TIME, SCM_NIL));
     t->type = SCM_FALSEP(type)? SCM_SYM_TIME_UTC : type;
+    return t;
+}
+
+
+ScmObj Scm_MakeTime(ScmObj type, long sec, long nsec)
+{
+    ScmTime *t = make_time_int(type);
+    SCM_SET_INT64_BY_LONG(t->sec, sec);
+    t->nsec = nsec;
+    return SCM_OBJ(t);
+}
+
+ScmObj Scm_MakeTime64(ScmObj type, ScmInt64 sec, long nsec)
+{
+    ScmTime *t = make_time_int(type);
     t->sec = sec;
     t->nsec = nsec;
     return SCM_OBJ(t);
@@ -823,6 +842,20 @@ void Scm_GetTimeOfDay(u_long *sec, u_long *usec)
 #endif /* !HAVE_GETTIMEOFDAY && !GAUCHE_WINDOWS */
 }
 
+/* Experimental.  This returns the microsecond-resolution time, wrapped
+   around the fixnum resolution.  In 32-bit architecture it's a bit more
+   than 1000seconds.  Good for micro-profiling, since this guarantees
+   no allocation.  Returned value can be negative. */
+long Scm_CurrentMicroseconds()
+{
+    u_long sec, usec;
+    Scm_GetTimeOfDay(&sec, &usec);
+    /* we ignore overflow */
+    usec += sec * 1000000;
+    usec &= (1UL<<(SCM_SMALL_INT_SIZE+1)) - 1;
+    if (usec > SCM_SMALL_INT_MAX) usec -= (1UL<<(SCM_SMALL_INT_SIZE+1));
+    return (long)usec;
+}
 
 ScmObj Scm_CurrentTime(void)
 {
@@ -836,14 +869,18 @@ ScmObj Scm_IntSecondsToTime(long sec)
     return Scm_MakeTime(SCM_SYM_TIME_UTC, sec, 0);
 }
 
+ScmObj Scm_Int64SecondsToTime(ScmInt64 sec)
+{
+    return Scm_MakeTime64(SCM_SYM_TIME_UTC, sec, 0);
+}
+
 ScmObj Scm_RealSecondsToTime(double sec)
 {
     double s, frac;
-    if (sec > (double)ULONG_MAX || sec < 0) {
-        Scm_Error("seconds out of range: %f", sec);
-    }
+    ScmInt64 secs;
     frac = modf(sec, &s);
-    return Scm_MakeTime(SCM_SYM_TIME_UTC, (long)s, (long)(frac * 1.0e9));
+    secs = Scm_DoubleToInt64(s);
+    return Scm_MakeTime64(SCM_SYM_TIME_UTC, secs, (long)(frac * 1.0e9));
 }
 
 static ScmObj time_type_get(ScmTime *t)
@@ -861,7 +898,7 @@ static void time_type_set(ScmTime *t, ScmObj val)
 
 static ScmObj time_sec_get(ScmTime *t)
 {
-    return Scm_MakeInteger(t->sec);
+    return Scm_MakeInteger64(t->sec);
 }
 
 static void time_sec_set(ScmTime *t, ScmObj val)
@@ -869,7 +906,7 @@ static void time_sec_set(ScmTime *t, ScmObj val)
     if (!SCM_REALP(val)) {
         Scm_Error("real number required, but got %S", val);
     }
-    t->sec = Scm_GetInteger(val);
+    t->sec = Scm_GetInteger64(val);
 }
 
 static ScmObj time_nsec_get(ScmTime *t)
@@ -917,9 +954,18 @@ time_t Scm_GetSysTime(ScmObj val)
 {
     if (SCM_TIMEP(val)) {
 #ifdef INTEGRAL_TIME_T
+        /* NB: If time_t is 64bit, we assume ScmUInt64 is int64_t.
+           So if we're emulating int64 then we reject conversion over 32bits*/
+#  if SCM_EMULATE_INT64
+        if (SCM_TIME(val)->sec.hi > 0) {
+            Scm_Error("time object is out of range for Unix time: %S", val);
+        }
+        return (time_t)(SCM_TIME(val)->sec.lo);
+#  else
         return (time_t)SCM_TIME(val)->sec;
+#  endif
 #else
-        return (time_t)((double)SCM_TIME(val)->sec +
+        return (time_t)(Scm_Int64ToDouble(SCM_TIME(val)->sec) +
                         (double)SCM_TIME(val)->nsec/1.0e9);
 #endif
     } else if (SCM_NUMBERP(val)) {
@@ -937,9 +983,9 @@ time_t Scm_GetSysTime(ScmObj val)
 ScmObj Scm_TimeToSeconds(ScmTime *t)
 {
     if (t->nsec) {
-        return Scm_MakeFlonum((double)t->sec + (double)t->nsec/1.0e9);
+        return Scm_MakeFlonum(Scm_Int64ToDouble(t->sec) + (double)t->nsec/1.0e9);
     } else {
-        return Scm_MakeIntegerFromUI(t->sec);
+        return Scm_MakeInteger64(t->sec);
     }
 }
 
@@ -949,13 +995,28 @@ struct timespec *Scm_GetTimeSpec(ScmObj t, struct timespec *spec)
 {
     if (SCM_FALSEP(t)) return NULL;
     if (SCM_TIMEP(t)) {
+#if SCM_EMULATE_INT64
+        /* if we don't have int64_t, it's very likely that timespec can't
+           handle 64bit time. */
+        if (SCM_TIME(t)->sec.hi > 0) {
+            Scm_Error("cannot convert Scheme time to struct timespec: out of range: %S", t);
+        }
+        spec->tv_sec = SCM_TIME(t)->sec.lo;
+        
+#else  /*!SCM_EMULATE_INT64*/
+        /* TODO: we might want to check if tv_sec can handle 64bit integer */
         spec->tv_sec = SCM_TIME(t)->sec;
+#endif /*!SCM_EMULATE_INT64*/
         spec->tv_nsec = SCM_TIME(t)->nsec;
     } else if (!SCM_REALP(t)) {
         Scm_Error("bad timeout spec: <time> object or real number is required, but got %S", t);
     } else {
         ScmTime *ct = SCM_TIME(Scm_CurrentTime());
+#if SCM_EMULATE_INT64
+        spec->tv_sec = ct->sec.lo;  /* TODO: 2038 */
+#else  /*!SCM_EMULATE_INT64*/
         spec->tv_sec = ct->sec;
+#endif /*!SCM_EMULATE_INT64*/
         spec->tv_nsec = ct->nsec;
         if (SCM_EXACTP(t)) {
             spec->tv_sec += Scm_GetUInteger(t);
@@ -1688,8 +1749,6 @@ ScmObj Scm_SysWait(ScmObj process, int options)
        process is integer and > 0  -> wait for specific pid
        process is #<win:process-handle> -> wait for specified process
        The common op is factored out in win_wait_for_handles. */
-    static int win_wait_for_handles(HANDLE *handles, int nhandles, int options,
-                                    int *status /*out*/);
     int r, status = 0;
 
     if (SCM_INTEGERP(process)) {
