@@ -1,7 +1,7 @@
 ;;;
 ;;; vminsn.scm - Virtual machine instruction definition
 ;;;
-;;;   Copyright (c) 2000-2005 Shiro Kawai, All rights reserved.
+;;;   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
 ;;; 
 ;;;   Redistribution and use in source and binary forms, with or without
 ;;;   modification, are permitted provided that the following conditions
@@ -30,7 +30,7 @@
 ;;;   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 ;;;   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 ;;;
-;;;  $Id: vminsn.scm,v 1.7 2006/01/10 09:59:46 shirok Exp $
+;;;  $Id: vminsn.scm,v 1.10 2007/04/18 02:26:45 shirok Exp $
 ;;;
 
 ;;; This file is processed by geninsn to produce a couple of C files:
@@ -423,6 +423,11 @@
 (define-insn NUMDIV2     0 none)        ; /
 (define-insn NEGATE      0 none)        ; -  (unary)
 
+(define-insn NUMIADD2    0 none)        ; +.
+(define-insn NUMISUB2    0 none)        ; -. (binary)
+(define-insn NUMIMUL2    0 none)        ; *.
+(define-insn NUMIDIV2    0 none)        ; /. (binary)
+
 (define-insn NUMADDI     1 none)        ; +, if one of op is small int
 (define-insn NUMSUBI     1 none)        ; -, if one of op is small int
 
@@ -493,3 +498,6 @@
 
 (define-insn PUSH-HANDLERS 0 none)      ; push dynamic handlers
 (define-insn POP-HANDLERS 0 none)       ; pop dynamic handlers
+
+;; experiment
+;(define-insn LREF-NUMADD2 2 none)
