@@ -1,7 +1,7 @@
 /*
  * write.c - writer
  *
- *   Copyright (c) 2000-2004 Shiro Kawai, All rights reserved.
+ *   Copyright (c) 2000-2007  Shiro Kawai  <shiro@acm.org>
  * 
  *   Redistribution and use in source and binary forms, with or without
  *   modification, are permitted provided that the following conditions
@@ -30,22 +30,23 @@
  *   NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
  *   SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  *
- *  $Id: write.c,v 1.59 2006/01/28 09:17:42 shirok Exp $
+ *  $Id: write.c,v 1.69 2007/08/24 23:55:44 shirok Exp $
  */
 
-#include <stdio.h>
-#include <ctype.h>
 #define LIBGAUCHE_BODY
 #include "gauche.h"
 #include "gauche/port.h"
 #include "gauche/builtin-syms.h"
-#include "gauche/code.h"        /* NB: for SCM_VM_INSNP -- remove this later */
+
+#include <ctype.h>
 
 static void write_walk(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void write_ss(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx);
 static void write_object(ScmObj obj, ScmPort *out, ScmWriteContext *ctx);
 static ScmObj write_object_fallback(ScmObj *args, int nargs, ScmGeneric *gf);
+static void format_write(ScmObj obj, ScmPort *port, ScmWriteContext *ctx,
+                         int sharedp);
 SCM_DEFINE_GENERIC(Scm_GenericWriteObject, write_object_fallback, NULL);
 
 /*============================================================
@@ -114,16 +115,6 @@ SCM_DEFINE_GENERIC(Scm_GenericWriteObject, write_object_fallback, NULL);
    (SCM_VM_RUNTIME_FLAG_IS_SET(Scm_VM(), SCM_CASE_FOLD)? \
     SCM_WRITE_CASE_FOLD:SCM_WRITE_CASE_NOFOLD)
 
-static inline int outlen(ScmPort *out)
-{
-    SCM_ASSERT(SCM_PORT_TYPE(out) == SCM_PORT_OSTR);
-    if (out->src.ostr.length < 0) {
-        return Scm_DStringSize(&out->src.ostr);
-    } else {
-        return out->src.ostr.length;
-    }
-}
-
 /*
  * Scm_Write - Standard Write.
  */
@@ -179,31 +170,36 @@ void Scm_Write(ScmObj obj, ScmObj p, int mode)
 int Scm_WriteLimited(ScmObj obj, ScmObj port, int mode, int width)
 {
     ScmWriteContext ctx;
+    ScmString *str;
     ScmObj out;
-    int nc;
+    int nc, sharedp = FALSE;
     
     if (!SCM_OPORTP(port))
         Scm_Error("output port required, but got %S", port);
     out = Scm_MakeOutputStringPort(TRUE);
+    SCM_PORT(out)->data = SCM_PORT(port)->data;
     ctx.mode = mode;
     ctx.flags = WRITE_LIMITED;
     ctx.limit = width;
     /* if case mode is not specified, use default taken from VM default */
     if (SCM_WRITE_CASE(&ctx) == 0) ctx.mode |= DEFAULT_CASE;
-    /* we don't need to lock out, for it is private. */
-    if (SCM_WRITE_MODE(&ctx) == SCM_WRITE_SHARED) {
-        write_ss(obj, SCM_PORT(out), &ctx);
-    } else {
-        write_ss_rec(obj, SCM_PORT(out), &ctx);
+    /* the walk pass does not produce any output, so we return immediately. */
+    if (SCM_PORT(port)->flags & SCM_PORT_WALKING) {
+        SCM_ASSERT(SCM_PAIRP(SCM_PORT(port)->data)&&SCM_HASH_TABLE_P(SCM_CDR(SCM_PORT(port)->data)));
+        write_walk(obj, SCM_PORT(port), &ctx);
+        return 0;               /* doesn't really matter */
     }
-    nc = outlen(SCM_PORT(out));
+    /* we don't need to lock out, for it is private. */
+    sharedp = SCM_WRITE_MODE(&ctx) == SCM_WRITE_SHARED;
+    format_write(obj, SCM_PORT(out), &ctx, sharedp);
+    str = SCM_STRING(Scm_GetOutputString(SCM_PORT(out), 0));
+    nc = SCM_STRING_BODY_LENGTH(SCM_STRING_BODY(str));
     if (nc > width) {
-        ScmObj sub = Scm_Substring(SCM_STRING(Scm_GetOutputString(SCM_PORT(out))),
-                                   0, width);
+        ScmObj sub = Scm_Substring(str, 0, width, FALSE);
         SCM_PUTS(sub, port);    /* this locks port */
         return -1;
     } else {
-        SCM_PUTS(Scm_GetOutputString(SCM_PORT(out)), port); /* this locks port */
+        SCM_PUTS(str, port);    /* this locks port */
         return nc;
     }
 }
@@ -214,6 +210,8 @@ int Scm_WriteLimited(ScmObj obj, ScmObj port, int mode, int width)
 
 int Scm_WriteCircular(ScmObj obj, ScmObj port, int mode, int width)
 {
+    ScmObj out;
+    ScmString *str;
     ScmWriteContext ctx;
     int nc;
 
@@ -230,28 +228,35 @@ int Scm_WriteCircular(ScmObj obj, ScmObj port, int mode, int width)
     ctx.ncirc = 0;
     ctx.table = SCM_HASH_TABLE(Scm_MakeHashTableSimple(SCM_HASH_EQ, 8));
 
-    if (width > 0) {
-        ScmObj out = Scm_MakeOutputStringPort(TRUE);
-        /* no need to lock out, for it is private */
-        write_ss(obj, SCM_PORT(out), &ctx);
-        nc = outlen(SCM_PORT(out));
-        if (nc > width) {
-            ScmObj sub = Scm_Substring(SCM_STRING(Scm_GetOutputString(SCM_PORT(out))),
-                                       0, width);
-            SCM_PUTS(sub, port); /* this locks port */
-            return -1;
-        } else {
-            SCM_PUTS(Scm_GetOutputString(SCM_PORT(out)), port); /* this locks port */
-            return nc;
-        }
-    } else {
+    if (width <= 0) {
         ScmVM *vm = Scm_VM();
         PORT_LOCK(SCM_PORT(port), vm);
         PORT_SAFE_CALL(SCM_PORT(port),
-                       write_ss(obj, SCM_PORT(port), &ctx));
+                       format_write(obj, SCM_PORT(port), &ctx, TRUE));
         PORT_UNLOCK(SCM_PORT(port));
+        return 0;
+    }
+
+    if (SCM_PORT(port)->flags & SCM_PORT_WALKING) {
+        SCM_ASSERT(SCM_PAIRP(SCM_PORT(port)->data)&&SCM_HASH_TABLE_P(SCM_CDR(SCM_PORT(port)->data)));
+        write_walk(obj, SCM_PORT(port), &ctx);
+        return 0;               /* doesn't really matter */
+    }
+
+    out = Scm_MakeOutputStringPort(TRUE);
+    SCM_PORT(out)->data = SCM_PORT(port)->data;
+    /* no need to lock out, for it is private */
+    format_write(obj, SCM_PORT(out), &ctx, TRUE);
+    str = SCM_STRING(Scm_GetOutputString(SCM_PORT(out),0));
+    nc = SCM_STRING_BODY_LENGTH(SCM_STRING_BODY(str));
+    if (nc > width) {
+        ScmObj sub = Scm_Substring(str, 0, width, FALSE);
+        SCM_PUTS(sub, port);    /* this locks port */
+        return -1;
+    } else {
+        SCM_PUTS(str, port);    /* this locks port */
+        return nc;
     }
-    return 0;
 }
 
 /*===================================================================
@@ -284,7 +289,8 @@ static void write_general(ScmObj obj, ScmPort *out, ScmWriteContext *ctx)
    may not return to VM immediately. */
 static void write_object(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
 {
-    Scm_Apply(SCM_OBJ(&Scm_GenericWriteObject), SCM_LIST2(obj, SCM_OBJ(port)));
+    Scm_ApplyRec(SCM_OBJ(&Scm_GenericWriteObject),
+                 SCM_LIST2(obj, SCM_OBJ(port)));
 }
 
 /* Default method for write-object */
@@ -407,7 +413,7 @@ static void write_ss_rec(ScmObj obj, ScmPort *port, ScmWriteContext *ctx)
     ScmHashTable *ht = NULL;
 
     if (ctx->flags & WRITE_LIMITED) {
-        if (outlen(port) >= ctx->limit) return;
+        if (port->src.ostr.length >= ctx->limit) return;
     }
 
     if (SCM_PAIRP(port->data) && SCM_HASH_TABLE_P(SCM_CDR(port->data))) {
@@ -653,17 +659,17 @@ static void format_sexp(ScmPort *out, ScmObj arg,
     if (minpad > 0 && !rightalign) {
         for (i=0; i<minpad; i++) Scm_PutcUnsafe(padchar, SCM_PORT(tmpout));
     }
-    tmpstr = SCM_STRING(Scm_GetOutputString(SCM_PORT(tmpout)));
+    tmpstr = SCM_STRING(Scm_GetOutputString(SCM_PORT(tmpout), 0));
 
     if (maxcol > 0 && nwritten < 0) {
         const char *s = Scm_GetStringContent(tmpstr, NULL, NULL, NULL), *e;
         if (dots && maxcol > 4) {
             e = Scm_StringPosition(tmpstr, maxcol-4);
-            Scm_PutzUnsafe(s, e-s, out);
+            Scm_PutzUnsafe(s, (int)(e-s), out);
             Scm_PutzUnsafe(" ...", 4, out);
         } else {
             e = Scm_StringPosition(tmpstr, maxcol);
-            Scm_PutzUnsafe(s, e-s, out);
+            Scm_PutzUnsafe(s, (int)(e-s), out);
         }
     } else {
         format_pad(out, tmpstr, mincol, colinc, padchar, rightalign);
@@ -1091,7 +1097,7 @@ static void vprintf_proc(ScmPort *out, const char *fmt, ScmObj args,
                     SCM_DSTRING_PUTB(&argbuf, c);
                     SCM_DSTRING_PUTB(&argbuf, 0);
                     snprintf(buf, SPBUFSIZ, Scm_DStringGetz(&argbuf),
-                             (void*)Scm_GetUInteger(val));
+                             (void*)(intptr_t)Scm_GetUInteger(val));
                     Scm_PutzUnsafe(buf, -1, out);
                     break;
                 }
@@ -1161,7 +1167,7 @@ static void vprintf_proc(ScmPort *out, const char *fmt, ScmObj args,
             break;
         }
         if (c == 0) {
-            Scm_Error("incomplete %-directive in format string: %s", fmt);
+            Scm_Error("incomplete %%-directive in format string: %s", fmt);
         }
     }
 }
@@ -1216,7 +1222,7 @@ void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, int sharedp)
             case 'p':
                 {
                     void *val = va_arg(ap, void *);
-                    SCM_APPEND1(h, t, Scm_MakeIntegerU((unsigned long)val));
+                    SCM_APPEND1(h, t, Scm_MakeIntegerU((u_long)(intptr_t)val));
                     break;
                 }
             case 'S':; case 'A':
@@ -1237,7 +1243,7 @@ void Scm_Vprintf(ScmPort *out, const char *fmt, va_list ap, int sharedp)
             break;
         }
         if (c == 0) {
-            Scm_Error("incomplete %-directive in format string: %s", fmt);
+            Scm_Error("incomplete %%-directive in format string: %s", fmt);
         }
     }
     /*
@@ -1266,6 +1272,33 @@ void Scm_PrintfShared(ScmPort *out, const char *fmt, ...)
     va_end(ap);
 }
 
+ScmObj Scm_Sprintf(const char *fmt, ...)
+{
+    ScmObj r;
+    va_list args;
+    va_start(args, fmt);
+    r = Scm_Vsprintf(fmt, args, FALSE);
+    va_end(args);
+    return r;
+}
+
+ScmObj Scm_SprintfShared(const char *fmt, ...)
+{
+    ScmObj r;
+    va_list args;
+    va_start(args, fmt);
+    r = Scm_Vsprintf(fmt, args, TRUE);
+    va_end(args);
+    return r;
+}
+
+ScmObj Scm_Vsprintf(const char *fmt, va_list ap, int sharedp)
+{
+    ScmObj ostr = Scm_MakeOutputStringPort(TRUE);
+    Scm_Vprintf(SCM_PORT(ostr), fmt, ap, sharedp);
+    return Scm_GetOutputString(SCM_PORT(ostr), 0);
+}
+
 /*
  * Initialization
  */
