@@ -1,6 +1,6 @@
 ;; test exception handling system 
 ;; this must come after primsyn, error, macro and object tests.
-;; $Id: exception.scm,v 1.10 2006/11/09 10:32:20 shirok Exp $
+;; $Id: exception.scm,v 1.12 2007/01/18 10:01:48 shirok Exp $
 
 (use gauche.test)
 (test-start "exceptions")
@@ -158,6 +158,13 @@
   (set! v5 (make-compound-condition v2 v3))
   (test* "v5" '(#t #t #t "V2" "a3" "b2")
          (list (c? v5) (c1? v5) (c2? v5) (c-x v5) (c1-a v5) (c2-b v5)))
+
+  (test* "error (&c1)" "ok"
+         (with-error-handler
+             (lambda (e)
+               (and (is-a? e &c1)
+                    (slot-ref e 'a)))
+           (lambda () (error &c1 :a "ok"))))
   )
 
 ;;--------------------------------------------------------------------
@@ -238,6 +245,52 @@
            (list x aaa))))
 
 
+;;--------------------------------------------------------------------
+(test-section "unwind-protect")
+
+(let ()
+  (define aaa '())
+  (define (foo thunk)
+    (dynamic-wind
+        (lambda () (push! aaa 'a))
+        (lambda () (push! aaa 'b) (thunk) (push! aaa 'c))
+        (lambda () (push! aaa 'd))))
+  (test* "unwind-protect (success)" '(e d c z b a)
+         (guard (e (else (push! aaa e) e))
+           (set! aaa '())
+           (unwind-protect
+            (foo (lambda () (push! aaa 'z)))
+            (push! aaa 'e))
+           aaa))
+  (test* "unwind-protect (raise)" '(boo e d b a)
+         (guard (e (else (push! aaa e) aaa))
+           (set! aaa '())
+           (unwind-protect
+            (foo (lambda () (raise 'boo)))
+            (push! aaa 'e))
+           aaa))
+  (test* "unwind-protect (error)" '(boo e d b a)
+         (guard (e (else (push! aaa 'boo) aaa))
+           (set! aaa '())
+           (unwind-protect
+            (foo (lambda () (error "boo")))
+            (push! aaa 'e))
+           aaa))
+
+  (test* "unwind-protect (restart)" '(e d c a d z b a)
+         (begin
+           (set! aaa '())
+           (let ((k #f))
+             (let/cc k1
+               (unwind-protect
+                (foo (lambda ()
+                       (let/cc k2
+                         (set! k k2) (push! aaa 'z) (k1 0))))
+                (push! aaa 'e)))
+             (when k (let ((k0 k)) (set! k #f) (k0 0))))
+           aaa))
+  )
+
 ;;--------------------------------------------------------------------
 (test-section "subtype")
 
