@@ -32,6 +32,48 @@
 (test "with-module (error)" *test-error*
       (lambda () (eval '(with-module MM 4) (interaction-environment))))
 
+(with-module M
+  (define + list)
+  (define if list))
+
+(test "with-module in head position" '(2 3)
+      (lambda ()
+        ((with-module M +) 2 3)))
+
+(test "with-module in head position" '(3 5 6)
+      (lambda ()
+        (with-module M
+          (if ((with-module scheme if) 2 3 4) 5 6))))
+
+(define-module MA
+  (export with-module)
+  (define-syntax with-module
+    (syntax-rules ()
+      ((_ a b) list))))
+
+(test "with-module in head position (shadowed)" '(1 2 3)
+      (lambda ()
+        (with-module MA
+          ((with-module x y) 1 2 3))))
+
+(define-module MB
+  (import MA)
+  (export oops)
+  (define-syntax oops
+    (syntax-rules ()
+      ((_ a) (define a 3)))))
+
+(test "with-module in head position (shadowed)" '(1 2 3)
+      (lambda ()
+        (with-module MB
+          ((with-module x y) 1 2 3))))
+
+(test "with-module in head position (in lambda body)" 6
+      (lambda ()
+        (let ((x 1))
+          ((with-module MB oops) x)
+          (+ x x))))
+
 ;;------------------------------------------------------------------
 ;; define-in-module
 
@@ -148,6 +190,22 @@
       (lambda ()
         (eval '(with-module V (extend Q MM)) (interaction-environment))))
 
+(test "global-variable-ref" 'gamma
+      (lambda ()
+        (global-variable-ref 'U 'c)))
+
+(test "global-variable-ref" *test-error*
+      (lambda ()
+        (global-variable-ref 'U 'e)))
+
+(test "global-variable-ref" 'huh?
+      (lambda ()
+        (global-variable-ref 'U 'e 'huh?)))
+
+(test "global-variable-ref" 'huh?
+      (lambda ()
+        (global-variable-ref 'U 'c 'huh? #t)))
+
 ;;------------------------------------------------------------------
 ;; creates modules on-the-fly
 
