@@ -2,7 +2,7 @@
 ;; testing regexp
 ;;
 
-;; $Id: regexp.scm,v 1.24 2007/08/26 08:16:05 shirok Exp $
+;; $Id: regexp.scm,v 1.27 2007/12/23 06:58:48 shirok Exp $
 
 (use gauche.test)
 (use srfi-1)
@@ -66,6 +66,25 @@
 (test-regexp-parse "(?<name>)(?<name>)\\k<name>"
                    '(0 #f (1 name) (2 name) (alt (backref . 2) (backref . 1))))
 
+;;-------------------------------------------------------------------------
+(test-section "regexp-writer")
+
+(define-syntax test-regexp-writer
+  (syntax-rules ()
+    ((_ exp pat)
+     (test* exp exp (write-to-string (string->regexp pat))))))
+
+(test-regexp-writer "#/abc/" "abc")
+(test-regexp-writer "#/a[\\d]/" "a[\\d]")
+(test-regexp-writer "#/a[a-z]/" "a[a-z]")
+(test-regexp-writer "#/a\\/b/"  "a/b")
+(test-regexp-writer "#/\\\\/"  "\\\\")
+(test-regexp-writer "#/ /"  "\x20")
+(test-regexp-writer "#/\\x20/"  "\\x20")
+
+(test* "#/abc/i" "#/abc/i"
+       (write-to-string (string->regexp "abc" :case-fold #t)))
+
 ;;-------------------------------------------------------------------------
 (test-section "compile")
 
@@ -113,6 +132,14 @@
 (test-regexp-compile "(?<name>)\\k<name>")
 (test-regexp-compile "(?<name>)(?<name>)\\k<name>")
 
+;;-------------------------------------------------------------------------
+(test-section "boundary")
+
+(test* "submatch #f" #f (rxmatch-start #f))
+(test* "submatch #f" #f (rxmatch-end #f))
+(test* "submatch #f" #f (rxmatch-substring #f))
+(test* "num-match #f" 0 (rxmatch-num-matches #f))
+
 ;;-------------------------------------------------------------------------
 (test-section "basics")
 
@@ -934,6 +961,12 @@
 (test* "regexp-replace" "abc|\\0|ghi"
        (regexp-replace #/def|DEF/ "abcdefghi" "|\\\\0|"))
 
+(test* "regexp-replace" "abc|def|ghi"
+       (regexp-replace #/(?<match>def|DEF)/ "abcdefghi" "|\\k<match>|"))
+
+(test* "regexp-replace" *test-error*
+       (regexp-replace #/(?<match>def|DEF)/ "abcdefghi" "|\\k<matchee>|"))
+
 (test* "regexp-replace" "abraabra**brabra**brabrabracadabrabrabra"
        (regexp-replace #/a((bra)+)cadabra/
                        "abraabraabrabracadabrabrabrabracadabrabrabra"
@@ -944,6 +977,11 @@
                            "abraabraabrabracadabrabrabrabracadabrabrabra"
                            "**\\1**"))
 
+(test* "regexp-replace-all" "abraabra**brabra**br**brabra**brabra"
+       (regexp-replace-all #/a(?<match>(bra)+)cadabra/
+                           "abraabraabrabracadabrabrabrabracadabrabrabra"
+                           "**\\k<match>**"))
+
 (test* "regexp-replace" "abfedhi"
        (regexp-replace #/c(.*)g/ "abcdefghi" 
                        (lambda (m)
