@@ -49,9 +49,9 @@ Content-Length: 4349
     ("content-length" "4349")
     ))
 
-(test* "rfc822-header->list" #t
+(test* "rfc822-read-headers" #t
        (equal? rfc822-header1-list
-               (rfc822-header->list (open-input-string rfc822-header1))))
+               (rfc822-read-headers (open-input-string rfc822-header1))))
 
 ;; token parsers
 (test* "rfc822-field->tokens (basic)"
@@ -119,6 +119,85 @@ Content-Length: 4349
 (test* "rfc822-parse-date (invalid)" '(#f #f #f #f #f #f #f #f)
        (receive r (rfc822-parse-date "Sun 2 Mar 2002") r))
 
+(test* "rfc822-invalid-header-field" #f
+       (rfc822-invalid-header-field "abcde"))
+(test* "rfc822-invalid-header-field" 'incomplete-string
+       (rfc822-invalid-header-field #*"abcde"))
+;; unicode literal doesn't work with none encoding
+(unless (eq? (gauche-character-encoding) 'none)
+  (test* "rfc822-invalid-header-field" 'bad-character
+         (rfc822-invalid-header-field "abc\u3030 def"))
+  )
+(test* "rfc822-invalid-header-field" 'bad-character
+       (rfc822-invalid-header-field "abc\x00 def"))
+(test* "rfc822-invalid-header-field" 'line-too-long
+       (rfc822-invalid-header-field (make-string 1000 #\a)))
+(test* "rfc822-invalid-header-field" 'line-too-long
+       (rfc822-invalid-header-field
+        (string-append (string-join (make-list 5 (make-string 78 #\a))
+                                    "\r\n ")
+                       (make-string 1000 #\a))))
+(test* "rfc822-invalid-header-field" 'stray-crlf
+       (rfc822-invalid-header-field
+        (string-join (make-list 5 (make-string 78 #\a)) "\r\n")))
+(test* "rfc822-invalid-header-field" 'stray-crlf
+       (rfc822-invalid-header-field "abc\ndef"))
+(test* "rfc822-invalid-header-field" 'stray-crlf
+       (rfc822-invalid-header-field "abc\rdef"))
+
+(test* "rfc822-write-headers"
+       "name: Shiro Kawai\r\n\
+        address: 1234 Lambda St.\r\n \
+        Higher Order Functions, HI, 99899\r\n\
+        registration-date: 2007-12-10\r\n\r\n"
+       (with-output-to-string
+         (cut rfc822-write-headers
+              '(("name" "Shiro Kawai")
+                ("address" "1234 Lambda St.\r\n Higher Order Functions, HI, 99899")
+                ("registration-date" "2007-12-10")))))
+
+(test* "rfc822-write-headers (ignore error)"
+       (make-list 2 "name: Shiro\x00Kawai\r\n\r\n")
+       (map (lambda (x)
+              (with-output-to-string
+                (cut rfc822-write-headers '(("name" "Shiro\x00Kawai"))
+                     :check x)))
+            '(#f :ignore)))
+
+(test* "rfc822-write-headers (continue)"
+       "x: A\r\nx: B\r\nx: C\r\n\r\n"
+       (call-with-output-string
+         (lambda (p)
+           (rfc822-write-headers '(("x" "A") ("x" "B")) :output p :continue #t)
+           (rfc822-write-headers '(("x" "C")) :output p))))
+
+(let-syntax ([test-reason
+              (syntax-rules ()
+                [(_ expect hdrs)
+                 (begin
+                   (test* (format "rfc822-write-headers error (~a)" expect)
+                          expect
+                          (guard (e (else (rxmatch-case (ref e'message)
+                                            [#/\(([\w-]+)\)/ (_ m) m]
+                                            [else e])))
+                            (with-output-to-string
+                              (cut rfc822-write-headers hdrs))))
+                   (test* (format "rfc822-write-headers handle (~a)" expect)
+                          (format "x-name: ~a\r\n\r\n" expect)
+                          (with-output-to-string
+                            (cut rfc822-write-headers hdrs
+                                 :check (lambda (name body reason)
+                                          (values #`"x-,name"
+                                                  (x->string reason))))))
+                   )])])
+
+  (test-reason "bad-character" '(("name" "Shiro\x00Kawai")))
+  (test-reason "incomplete-string" '(("name" #*"Shiro Kawai")))
+  (test-reason "stray-crlf" '(("name" "Shiro\nKawai")))
+  (test-reason "line-too-long" `(("name" ,(make-string 1000 #\a))))
+  )
+        
+
 ;;--------------------------------------------------------------------
 (test-section "rfc.base64")
 (use rfc.base64)
@@ -136,6 +215,40 @@ Content-Length: 4349
 (test* "encode" "OTNiamFl" (base64-encode-string "93bjae"))
 (test* "encode" "QkFSMGVyOQ==" (base64-encode-string "BAR0er9"))
 
+(test* "encode w/ line width (default)"
+       "MDEyMzQ1Njc4OTAxMjM0NTY3ODkwMTIzNDU2Nzg5MDEyMzQ1Njc4OTAxMjM0NTY3ODkwMTIzNDU2\n"
+       (base64-encode-string "012345678901234567890123456789012345678901234567890123456"))
+(test* "encode w/ line width 10, e1"
+       "MDEyMzQ1Ng\n=="
+       (base64-encode-string "0123456" :line-width 10))
+(test* "encode w/ line width 11, e1"
+       "MDEyMzQ1Ng=\n="
+       (base64-encode-string "0123456" :line-width 11))
+(test* "encode w/ line width 12, e1"
+       "MDEyMzQ1Ng==\n"
+       (base64-encode-string "0123456" :line-width 12))
+(test* "encode w/ line width 11, e2"
+       "MDEyMzQ1Njc\n="
+       (base64-encode-string "01234567" :line-width 11))
+(test* "encode w/ line width 12, e2"
+       "MDEyMzQ1Njc=\n"
+       (base64-encode-string "01234567" :line-width 12))
+(test* "encode w/ line width 4"
+       "MDEy\nMzQ=\n"
+       (base64-encode-string "01234" :line-width 4))
+(test* "encode w/ line width 3"
+       "MDE\nyMz\nQ="
+       (base64-encode-string "01234" :line-width 3))
+(test* "encode w/ line width 2"
+       "MD\nEy\nMz\nQ=\n"
+       (base64-encode-string "01234" :line-width 2))
+(test* "encode w/ line width 1"
+       "M\nD\nE\ny\nM\nz\nQ\n=\n"
+       (base64-encode-string "01234" :line-width 1))
+(test* "encode w/ line width 0"
+       "MDEyMzQ="
+       (base64-encode-string "01234" :line-width 0))
+
 (test* "decode" "" (base64-decode-string ""))
 (test* "decode" "a" (base64-decode-string "YQ=="))
 (test* "decode" "a" (base64-decode-string "YQ="))
@@ -165,6 +278,29 @@ Content-Length: 4349
 (test* "encode (soft line break)"
        "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abc=\r\ndefghij0123456789abcdefghij"
        (quoted-printable-encode-string "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij"))
+(test* "encode (soft line break w/line-width)"
+       "0123456789abcdefg=\r\nhij0123456789abcd=\r\nefghij"
+       (quoted-printable-encode-string
+        "0123456789abcdefghij0123456789abcdefghij"
+        :line-width 20))
+(test* "encode (soft line break w/line-width)"
+       "0123456789abcdef=3D=\r\nghij0123456789a=3D=\r\n=3Dbcdefghij"
+       (quoted-printable-encode-string
+        "0123456789abcdef=ghij0123456789a==bcdefghij"
+        :line-width 20))
+(test* "encode (soft line break w/line-width lower bound)"
+       "a=\r\n=3F=\r\nb"
+       (quoted-printable-encode-string "a?b" :line-width 4))
+(test* "encode (no line break)"
+       "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij"
+       (quoted-printable-encode-string "0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij0123456789abcdefghij"
+                                       :line-width #f))
+(test* "encode (hard line break)"
+       "a\r\nb\r\nc\r\n"
+       (quoted-printable-encode-string "a\rb\nc\r\n"))
+(test* "encode (binary)"
+       "a=0Db=0Ac=0D=0A"
+       (quoted-printable-encode-string "a\rb\nc\r\n" :binary #t))
 
 (test* "decode" "\x01\x08abcde=\r\n"
        (quoted-printable-decode-string "=01=08abc=64=65=3D\r\n"))
@@ -306,7 +442,6 @@ Content-Length: 4349
 (test* "mime-parse-content-type"
        '("text" "plain" ("zzz" . "yyy") ("xxx" . "www"))
        (mime-parse-content-type " text/plain ;zzz=\"yyy\"; xxx = www (AAA)"))
-
 (test* "mime-parse-content-type"
        '("multipart" "alternative"
          ("boundary" . "=_alternative 006EBAA488256DF0_="))
@@ -314,22 +449,110 @@ Content-Length: 4349
         "multipart/alternative; boundary=\"=_alternative 006EBAA488256DF0_=\"")
        )
 
+#| ; TODO: enable these after rfc2231 support
+(test* "mime-parse-content-type (rfc2231 encode)"
+       '("application" "x-stuff"
+         ("x" . "y")
+         ("title" . "This is ***fun***")
+         ("url" . "http://www.example.com/"))
+       (mime-parse-content-type
+        "application/x-stuff; x=y; title*=us-ascii'en-us'This%20is%20%2A%2A%2Afun%2A%2A%2A; url=\"http://www.example.com/\""))
+
+(test* "mime-parse-content-type (rfc2231 concatenation)"
+       '("message" "external-body"
+         ("access-type" . "URL")
+         ("url" . "ftp://cs.utk.edu/pub/moore/bulk-mailer/bulk-mailer.tar"))
+       (mime-parse-content-type
+       "Content-Type: message/external-body; access-type=URL; \
+         URL*0=\"ftp://\"; \
+         URL*1=\"cs.utk.edu/pub/moore/bulk-mailer/bulk-mailer.tar\""))
+
+(test* "mime-parse-content-type (rfc2231 concatenation and encode)"
+       '("application" "x-stuff"
+         ("title" . "This is even more ***fun*** isn't it!")
+         ("a" . "b"))
+       (mime-parse-content-type
+        "Content-Type: application/x-stuff; \
+           title*0*=us-ascii'en'This%20is%20even%20more%20 \
+           title*1*=%2A%2A%2Afun%2A%2A%2A%20 \
+           title*2=\"isn't it!\"; \
+           a=b"))
+|#
+
+(test* "mime-encode-text (pass-through)" "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod\r\n tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim\r\n veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea\r\n commodo consequat. Duis aute irure dolor in reprehenderit in voluptate\r\n velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat\r\n cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id\r\n est laborum."
+       (mime-encode-text "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."))
+(test* "mime-encode-text (pass-through, nonbreak)" "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
+       (mime-encode-text "Lorem ipsum dolor sit amet, consectetur adipisicing elit, sed do eiusmod tempor incididunt ut labore et dolore magna aliqua. Ut enim ad minim veniam, quis nostrud exercitation ullamco laboris nisi ut aliquip ex ea commodo consequat. Duis aute irure dolor in reprehenderit in voluptate velit esse cillum dolore eu fugiat nulla pariatur. Excepteur sint occaecat cupidatat non proident, sunt in culpa qui officia deserunt mollit anim id est laborum."
+                         :line-width #f))
+(test* "mime-encode-text (pass-through, forced line break)" "Loremipsumdolorsitamet,consecteturadipisicingelit,seddoeiusmodtemporincididu\r\n ntutlaboreetdoloremagnaaliqua."
+       (mime-encode-text "Loremipsumdolorsitamet,consecteturadipisicingelit,seddoeiusmodtemporincididuntutlaboreetdoloremagnaaliqua."))
+
+;; unicode literal doesn't work with none encoding
+(unless (eq? (gauche-character-encoding) 'none)
+  (test* "mime-encode-text" "=?utf-8?B?zrvjga7lroflrpnjgbjjgojjgYbjgZPjgZ0=?=\r\n =?utf-8?B?44CCV2VsY29tZSB0byDOuy1zcGFjZQ==?="
+         (mime-encode-text "\u03bb\u306e\u5b87\u5b99\u3078\u3088\u3046\u3053\u305d\u3002\u0057\u0065\u006c\u0063\u006f\u006d\u0065\u0020\u0074\u006f\u0020\u03bb\u002d\u0073\u0070\u0061\u0063\u0065" :line-width 50))
+  (test* "mime-encode-text (nobreak)" "=?utf-8?B?zrvjga7lroflrpnjgbjjgojjgYbjgZPjgZ3jgIJXZWxjb21lIHRvIM67LXNwYWNl?="
+         (mime-encode-text "\u03bb\u306e\u5b87\u5b99\u3078\u3088\u3046\u3053\u305d\u3002\u0057\u0065\u006c\u0063\u006f\u006d\u0065\u0020\u0074\u006f\u0020\u03bb\u002d\u0073\u0070\u0061\u0063\u0065" :line-width #f))
+  )
+
 (use gauche.charconv)
+
 (when (ces-conversion-supported? "iso-8859-1" #f)
   (test* "mime-decode-word" "this is some text"
-         (mime-decode-word "=?iso-8859-1?q?this=20is=20some=20text?=")))
+         (mime-decode-word "=?iso-8859-1?q?this=20is=20some=20text?="))
+  (test* "mime-decode-text" "this is some text"
+         (mime-decode-text
+          "=?iso-8859-1?q?this=20is?= =?iso-8859-1?q?some=20text?="))
+  (test* "mime-decode-text" "this is some text"
+         (mime-decode-text
+          "=?iso-8859-1?q?this=20is?= some=?iso-8859-1?q?=20text?="))
+
+  (test* "mime-encode-word" "=?iso-8859-1?Q?this=20is=20some=20text?="
+         (mime-encode-word "this is some text" :charset 'iso-8859-1
+                           :transfer-encoding 'quoted-printable))
+  (test* "mime-encode-text" "=?iso-8859-1?B?VGhlIHF1aWNr?=\r\n =?iso-8859-1?B?IGJyb3duIGZv?=\r\n =?iso-8859-1?B?eCBqdW1wcyBv?=\r\n =?iso-8859-1?B?dmVyIHRoZSBs?=\r\n =?iso-8859-1?B?YXp5IGRvZw==?="
+         (mime-encode-text "The quick brown fox jumps over the lazy dog"
+                           :charset 'iso-8859-1 :force #t
+                           :line-width 30))
+  (test* "mime-encode-text" "\r\n =?iso-8859-1?B?VGhlIHF1aWNrIGJyb3du?=\r\n =?iso-8859-1?B?IGZveCBqdW1wcyBvdmVy?=\r\n =?iso-8859-1?B?IHRoZSBsYXp5IGRvZw==?="
+         (mime-encode-text "The quick brown fox jumps over the lazy dog"
+                           :charset 'iso-8859-1 :force #t
+                           :line-width 40
+                           :start-column 20))
+  )
+  
 (when (ces-conversion-supported? "us-ascii" #f)
   (test* "mime-decode-word" "Keith_Moore"
-         (mime-decode-word "=?US-ASCII?Q?Keith_Moore?=")))
-(when (and (memq (gauche-character-encoding) '(euc-jp sjis utf8))
+         (mime-decode-word "=?US-ASCII?Q?Keith_Moore?="))
+  (test* "mime-decode-word" "Keith_Moore"
+         (mime-decode-word "=?US-ASCII?B?S2VpdGhfTW9vcmU=?="))
+  (test* "mime-decode-text" "Keith/Moore"
+         (mime-decode-text "=?US-ASCII?B?S2VpdGg=?=/=?US-ASCII?Q?Moore?="))
+
+  (test* "mime-encode-text" "=?us-ascii?B?VGhlIHF1aWNr?=\r\n =?us-ascii?B?IGJyb3duIGZv?=\r\n =?us-ascii?B?eCBqdW1wcyBv?=\r\n =?us-ascii?B?dmVyIHRoZSBs?=\r\n =?us-ascii?B?YXp5IGRvZw==?="
+         (mime-encode-text "The quick brown fox jumps over the lazy dog"
+                           :charset 'us-ascii :force #t
+                           :line-width 30))
+  )
+                           
+(when (and (memq (gauche-character-encoding) '(euc-jp sjis utf-8))
            (ces-conversion-supported? "iso-2022-jp" #f))
   (test* "mime-decode-word" "\u5ddd\u5408 \u53f2\u6717"
          (mime-decode-word "=?ISO-2022-JP?B?GyRCQG45ZxsoQiAbJEI7S08vGyhC?="))
+  (test* "mime-decode-text" "(\u5ddd\u5408 \u53f2\u6717)"
+         (mime-decode-text "(=?iso-2022-jp?b?GyRCQG45ZxsoQg==?= =?iso-2022-jp?b?GyRCO0tPLxsoQg==?=)"))
+  (test* "mime-encode-word" "=?iso-2022-jp?B?GyRCQG45ZxsoQiAbJEI7S08vGyhC?="
+         (mime-encode-word "\u5ddd\u5408 \u53f2\u6717"
+                           :charset 'iso-2022-jp))
   )
+
 ;; this tests whether illegal input sequence is handled gracefully
-(when (memq (gauche-character-encoding) '(euc-jp sjis utf8))
+(when (memq (gauche-character-encoding) '(euc-jp sjis utf-8))
   (test* "mime-decode-word" "=?ISO-2022-JP?B?GyRCJDkbKBsoQg==?="
-         (mime-decode-word "=?ISO-2022-JP?B?GyRCJDkbKBsoQg==?=")))
+         (mime-decode-word "=?ISO-2022-JP?B?GyRCJDkbKBsoQg==?="))
+  (test* "mime-decode-text" "(=?ISO-2022-JP?B?GyRCJDkbKBsoQg==?=)"
+         (mime-decode-text "(=?ISO-2022-JP?B?GyRCJDkbKBsoQg==?=)"))
+  )
 
 ;; NB: this assumes the test is run either under src/ or test/
 (define (mime-message-tester num headers)
@@ -340,7 +563,7 @@ Content-Length: 4349
       (lambda (inp)
         (let* ((title (read-line inp)) ;; test title
                (expl  (read-line inp)) ;; explanation (ignored)
-               (headers (or headers (rfc822-header->list inp))))
+               (headers (or headers (rfc822-read-headers inp))))
           (test* #`"mime-parse-message (,|num| - ,|title|)"
                  res
                  (and (equal? (mime-parse-version
@@ -496,7 +719,7 @@ Content-Length: 4349
            (request-line (read-line in)))
       (rxmatch-if (#/^(\S+) (\S+) HTTP\/1\.1$/ request-line)
           (#f method request-uri)
-        (let* ((headers (rfc822-header->list in))
+        (let* ((headers (rfc822-read-headers in))
                (bodylen
                 (cond ((assoc-ref headers "content-length")
                        => (lambda (e) (string->number (car e))))
@@ -533,9 +756,9 @@ Content-Length: 4349
          (read-line pipe-in)
          (close-input-port pipe-in))))
 
-(test* "http-get" '(("method" "GET")
+(test* "http-get" `(("method" "GET")
                     ("request-uri" "/get")
-                    ("host" "localhost")
+                    ("host" ,#`"localhost:,*http-port*")
                     ("my-header" "foo")
                     ("request-body" ""))
        (receive (code headers body)
@@ -575,10 +798,10 @@ Content-Length: 4349
               (equal? headers '(("content-type" "text/plain")))
               (not body))))
 
-(test* "http-post" '(("method" "POST")
+(test* "http-post" `(("method" "POST")
                      ("request-uri" "/post")
                      ("content-length" "4")
-                     ("host" "localhost")
+                     ("host" ,#`"localhost:,*http-port*")
                      ("request-body" "data"))
        (receive (code headers body)
            (http-post #`"localhost:,*http-port*" "/post" "data")
